@@ -3,7 +3,8 @@
  * CONFIG.C - Configuration input and verification routines for Icinga
  *
  * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
- * Copyright (c) 2009-2010 Icinga Development Team
+ * Copyright (c) 2009-2011 Nagios Core Development Team and Community Contributors
+ * Copyright (c) 2009-2011 Icinga Development Team
  *
  * License:
  *
@@ -45,7 +46,6 @@ extern char	*p1_file;
 extern char     *nagios_user;
 extern char     *nagios_group;
 
-extern char     *macro_x[MACRO_X_COUNT];
 extern char     *macro_user[MAX_USER_MACROS];
 
 extern char     *global_host_event_handler;
@@ -207,6 +207,7 @@ extern serviceescalation *serviceescalation_list;
 extern servicedependency *servicedependency_list;
 extern hostdependency   *hostdependency_list;
 extern hostescalation   *hostescalation_list;
+extern module           *module_list;
 
 extern host		**host_hashlist;
 extern service		**service_hashlist;
@@ -219,11 +220,15 @@ extern char             *debug_file;
 extern int              debug_level;
 extern int              debug_verbosity;
 extern unsigned long    max_debug_file_size;
+
+extern int              allow_empty_hostgroup_assignment;
+
 /* make sure gcc3 won't hit here */
 #ifndef GCCTOOOLD
 extern int              event_profiling_enabled;
 #endif
 
+int dummy;	/* reduce compiler warnings */
 
 /******************************************************************/
 /************** CONFIGURATION INPUT FUNCTIONS *********************/
@@ -269,7 +274,9 @@ int read_main_config_file(char *main_config_file){
 	char *modptr=NULL;
 	char *argptr=NULL;
 	DIR *tmpdir=NULL;
+	icinga_macros *mac;
 
+	mac = get_global_macros();
 
 	/* open the config file for reading */
 	if((thefile=mmap_fopen(main_config_file))==NULL){
@@ -278,9 +285,9 @@ int read_main_config_file(char *main_config_file){
 		}
 
 	/* save the main config file macro */
-	my_free(macro_x[MACRO_MAINCONFIGFILE]);
-	if((macro_x[MACRO_MAINCONFIGFILE]=(char *)strdup(main_config_file)))
-		strip(macro_x[MACRO_MAINCONFIGFILE]);
+	my_free(mac->x[MACRO_MAINCONFIGFILE]);
+	if((mac->x[MACRO_MAINCONFIGFILE]=(char *)strdup(main_config_file)))
+		strip(mac->x[MACRO_MAINCONFIGFILE]);
 
 	/* process all lines in the config file */
 	while(1){
@@ -304,24 +311,24 @@ int read_main_config_file(char *main_config_file){
 
 		/* get the variable name */
 		if((temp_ptr=my_strtok(input,"="))==NULL){
-			asprintf(&error_message,"NULL variable");
+			dummy=asprintf(&error_message,"NULL variable");
 			error=TRUE;
 			break;
 			}
 		if((variable=(char *)strdup(temp_ptr))==NULL){
-			asprintf(&error_message,"malloc() error");
+			dummy=asprintf(&error_message,"malloc() error");
 			error=TRUE;
 			break;
 		        }
 
 		/* get the value */
 		if((temp_ptr=my_strtok(NULL,"\n"))==NULL){
-			asprintf(&error_message,"NULL value");
+			dummy=asprintf(&error_message,"NULL value");
 			error=TRUE;
 			break;
 			}
 		if((value=(char *)strdup(temp_ptr))==NULL){
-			asprintf(&error_message,"malloc() error");
+			dummy=asprintf(&error_message,"malloc() error");
 			error=TRUE;
 			break;
 		        }
@@ -333,8 +340,8 @@ int read_main_config_file(char *main_config_file){
 		if(!strcmp(variable,"resource_file")){
 
 			/* save the macro */
-			my_free(macro_x[MACRO_RESOURCEFILE]);
-			macro_x[MACRO_RESOURCEFILE]=(char *)strdup(value);
+			my_free(mac->x[MACRO_RESOURCEFILE]);
+			mac->x[MACRO_RESOURCEFILE]=(char *)strdup(value);
 
 			/* process the resource file */
 			read_resource_file(value);
@@ -343,7 +350,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Log file is too long");
+				dummy=asprintf(&error_message,"Log file is too long");
 				error=TRUE;
 				break;
 				}
@@ -352,8 +359,8 @@ int read_main_config_file(char *main_config_file){
 			log_file=(char *)strdup(value);
 
 			/* save the macro */
-			my_free(macro_x[MACRO_LOGFILE]);
-			macro_x[MACRO_LOGFILE]=(char *)strdup(log_file);
+			my_free(mac->x[MACRO_LOGFILE]);
+			mac->x[MACRO_LOGFILE]=(char *)strdup(log_file);
 			}
 
 		else if(!strcmp(variable,"debug_level"))
@@ -365,7 +372,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"debug_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Debug log file is too long");
+				dummy=asprintf(&error_message,"Debug log file is too long");
 				error=TRUE;
 				break;
 				}
@@ -380,7 +387,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"command_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Command file is too long");
+				dummy=asprintf(&error_message,"Command file is too long");
 				error=TRUE;
 				break;
 				}
@@ -389,14 +396,14 @@ int read_main_config_file(char *main_config_file){
 			command_file=(char *)strdup(value);
 
 			/* save the macro */
-			my_free(macro_x[MACRO_COMMANDFILE]);
-			macro_x[MACRO_COMMANDFILE]=(char *)strdup(value);
+			my_free(mac->x[MACRO_COMMANDFILE]);
+			mac->x[MACRO_COMMANDFILE]=(char *)strdup(value);
 			}
 
 		else if(!strcmp(variable,"temp_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Temp file is too long");
+				dummy=asprintf(&error_message,"Temp file is too long");
 				error=TRUE;
 				break;
 				}
@@ -405,20 +412,20 @@ int read_main_config_file(char *main_config_file){
 			temp_file=(char *)strdup(value);
 
 			/* save the macro */
-			my_free(macro_x[MACRO_TEMPFILE]);
-			macro_x[MACRO_TEMPFILE]=(char *)strdup(temp_file);
+			my_free(mac->x[MACRO_TEMPFILE]);
+			mac->x[MACRO_TEMPFILE]=(char *)strdup(temp_file);
 			}
 
 		else if(!strcmp(variable,"temp_path")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Temp path is too long");
+				dummy=asprintf(&error_message,"Temp path is too long");
 				error=TRUE;
 				break;
 				}
 
 			if((tmpdir=opendir((char *)value))==NULL){
-				asprintf(&error_message,"Temp path is not a valid directory");
+				dummy=asprintf(&error_message,"Temp path is not a valid directory");
 				error=TRUE;
 				break;
 				}
@@ -433,20 +440,20 @@ int read_main_config_file(char *main_config_file){
 			        }
 
 			/* save the macro */
-			my_free(macro_x[MACRO_TEMPPATH]);
-			macro_x[MACRO_TEMPPATH]=(char *)strdup(temp_path);
+			my_free(mac->x[MACRO_TEMPPATH]);
+			mac->x[MACRO_TEMPPATH]=(char *)strdup(temp_path);
 			}
 
 		else if(!strcmp(variable,"check_result_path")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Check result path is too long");
+				dummy=asprintf(&error_message,"Check result path is too long");
 				error=TRUE;
 				break;
 				}
 
 			if((tmpdir=opendir((char *)value))==NULL){
-				asprintf(&error_message,"Check result path is not a valid directory");
+				dummy=asprintf(&error_message,"Check result path is not a valid directory");
 				error=TRUE;
 				break;
 				}
@@ -467,7 +474,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"lock_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Lock file is too long");
+				dummy=asprintf(&error_message,"Lock file is too long");
 				error=TRUE;
 				break;
 				}
@@ -509,21 +516,21 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"admin_email")){
 
 			/* save the macro */
-			my_free(macro_x[MACRO_ADMINEMAIL]);
-			macro_x[MACRO_ADMINEMAIL]=(char *)strdup(value);
+			my_free(mac->x[MACRO_ADMINEMAIL]);
+			mac->x[MACRO_ADMINEMAIL]=(char *)strdup(value);
 		        }
 
 		else if(!strcmp(variable,"admin_pager")){
 
 			/* save the macro */
-			my_free(macro_x[MACRO_ADMINPAGER]);
-			macro_x[MACRO_ADMINPAGER]=(char *)strdup(value);
+			my_free(mac->x[MACRO_ADMINPAGER]);
+			mac->x[MACRO_ADMINPAGER]=(char *)strdup(value);
 		        }
 
         else if(!strcmp(variable,"use_daemon_log")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_daemon_log");
+				dummy=asprintf(&error_message,"Illegal value for use_daemon_log");
 				error=TRUE;
 				break;
 				}
@@ -534,7 +541,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_syslog")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_syslog");
+				dummy=asprintf(&error_message,"Illegal value for use_syslog");
 				error=TRUE;
 				break;
 				}
@@ -545,7 +552,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_syslog_local_facility")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_syslog_local_facility");
+				dummy=asprintf(&error_message,"Illegal value for use_syslog_local_facility");
 				error=TRUE;
 				break;
 				}
@@ -556,7 +563,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"syslog_local_facility")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'7'){
-				asprintf(&error_message,"Illegal value for syslog_local_facility");
+				dummy=asprintf(&error_message,"Illegal value for syslog_local_facility");
 				error=TRUE;
 				break;
 				}
@@ -567,7 +574,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_notifications")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_notifications");
+				dummy=asprintf(&error_message,"Illegal value for log_notifications");
 				error=TRUE;
 				break;
 			        }
@@ -578,7 +585,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_service_retries")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_service_retries");
+				dummy=asprintf(&error_message,"Illegal value for log_service_retries");
 				error=TRUE;
 				break;
 			        }
@@ -589,7 +596,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_host_retries")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_host_retries");
+				dummy=asprintf(&error_message,"Illegal value for log_host_retries");
 				error=TRUE;
 				break;
 			        }
@@ -600,7 +607,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_event_handlers")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_event_handlers");
+				dummy=asprintf(&error_message,"Illegal value for log_event_handlers");
 				error=TRUE;
 				break;
 			        }
@@ -611,7 +618,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_external_commands")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_external_commands");
+				dummy=asprintf(&error_message,"Illegal value for log_external_commands");
 				error=TRUE;
 				break;
 			        }
@@ -622,7 +629,7 @@ int read_main_config_file(char *main_config_file){
                 else if(!strcmp(variable,"log_external_commands_user")){
 
                         if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-                                asprintf(&error_message,"Illegal value for log_external_commands_user");
+                                dummy=asprintf(&error_message,"Illegal value for log_external_commands_user");
                                 error=TRUE;
                                 break;
                                 }
@@ -633,7 +640,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_passive_checks")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_passive_checks");
+				dummy=asprintf(&error_message,"Illegal value for log_passive_checks");
 				error=TRUE;
 				break;
 			        }
@@ -644,7 +651,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_long_plugin_output")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_long_plugin_output");
+				dummy=asprintf(&error_message,"Illegal value for log_long_plugin_output");
 				error=TRUE;
 				break;
 			        }
@@ -655,7 +662,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_initial_states")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_initial_states");
+				dummy=asprintf(&error_message,"Illegal value for log_initial_states");
 				error=TRUE;
 				break;
 			        }
@@ -666,7 +673,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_current_states")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for log_current_states");
+				dummy=asprintf(&error_message,"Illegal value for log_current_states");
 				error=TRUE;
 				break;
 			        }
@@ -677,7 +684,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"retain_state_information")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for retain_state_information");
+				dummy=asprintf(&error_message,"Illegal value for retain_state_information");
 				error=TRUE;
 				break;
 			        }
@@ -689,7 +696,7 @@ int read_main_config_file(char *main_config_file){
 
 			retention_update_interval=atoi(value);
 			if(retention_update_interval<0){
-				asprintf(&error_message,"Illegal value for retention_update_interval");
+				dummy=asprintf(&error_message,"Illegal value for retention_update_interval");
 				error=TRUE;
 				break;
 			        }
@@ -698,7 +705,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_retained_program_state")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_retained_program_state");
+				dummy=asprintf(&error_message,"Illegal value for use_retained_program_state");
 				error=TRUE;
 				break;
 			        }
@@ -709,7 +716,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_retained_scheduling_info")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_retained_scheduling_info");
+				dummy=asprintf(&error_message,"Illegal value for use_retained_scheduling_info");
 				error=TRUE;
 				break;
 			        }
@@ -722,7 +729,7 @@ int read_main_config_file(char *main_config_file){
 			retention_scheduling_horizon=atoi(value);
 
 			if(retention_scheduling_horizon<=0){
-				asprintf(&error_message,"Illegal value for retention_scheduling_horizon");
+				dummy=asprintf(&error_message,"Illegal value for retention_scheduling_horizon");
 				error=TRUE;
 				break;
 			        }
@@ -752,7 +759,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"obsess_over_services")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for obsess_over_services");
+				dummy=asprintf(&error_message,"Illegal value for obsess_over_services");
 				error=TRUE;
 				break;
 			        }
@@ -763,7 +770,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"obsess_over_hosts")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for obsess_over_hosts");
+				dummy=asprintf(&error_message,"Illegal value for obsess_over_hosts");
 				error=TRUE;
 				break;
 			        }
@@ -774,7 +781,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"translate_passive_host_checks")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for translate_passive_host_checks");
+				dummy=asprintf(&error_message,"Illegal value for translate_passive_host_checks");
 				error=TRUE;
 				break;
 			        }
@@ -790,7 +797,7 @@ int read_main_config_file(char *main_config_file){
 			service_check_timeout=atoi(value);
 
 			if(service_check_timeout<=0){
-				asprintf(&error_message,"Illegal value for service_check_timeout");
+				dummy=asprintf(&error_message,"Illegal value for service_check_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -807,7 +814,7 @@ int read_main_config_file(char *main_config_file){
 			else if(!strcmp(value,"u"))
 				service_check_timeout_state=STATE_UNKNOWN;
 			else{
-				asprintf(&error_message,"Illegal value for service_check_timeout_state");
+				dummy=asprintf(&error_message,"Illegal value for service_check_timeout_state");
 				error=TRUE;
 				break;
 			        }
@@ -818,7 +825,7 @@ int read_main_config_file(char *main_config_file){
 			host_check_timeout=atoi(value);
 
 			if(host_check_timeout<=0){
-				asprintf(&error_message,"Illegal value for host_check_timeout");
+				dummy=asprintf(&error_message,"Illegal value for host_check_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -829,7 +836,7 @@ int read_main_config_file(char *main_config_file){
 			event_handler_timeout=atoi(value);
 
 			if(event_handler_timeout<=0){
-				asprintf(&error_message,"Illegal value for event_handler_timeout");
+				dummy=asprintf(&error_message,"Illegal value for event_handler_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -840,7 +847,7 @@ int read_main_config_file(char *main_config_file){
 			notification_timeout=atoi(value);
 
 			if(notification_timeout<=0){
-				asprintf(&error_message,"Illegal value for notification_timeout");
+				dummy=asprintf(&error_message,"Illegal value for notification_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -851,7 +858,7 @@ int read_main_config_file(char *main_config_file){
 			ocsp_timeout=atoi(value);
 
 			if(ocsp_timeout<=0){
-				asprintf(&error_message,"Illegal value for ocsp_timeout");
+				dummy=asprintf(&error_message,"Illegal value for ocsp_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -862,7 +869,7 @@ int read_main_config_file(char *main_config_file){
 			ochp_timeout=atoi(value);
 
 			if(ochp_timeout<=0){
-				asprintf(&error_message,"Illegal value for ochp_timeout");
+				dummy=asprintf(&error_message,"Illegal value for ochp_timeout");
 				error=TRUE;
 				break;
 			        }
@@ -871,7 +878,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_agressive_host_checking") || !strcmp(variable,"use_aggressive_host_checking")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_aggressive_host_checking");
+				dummy=asprintf(&error_message,"Illegal value for use_aggressive_host_checking");
 				error=TRUE;
 				break;
 			        }
@@ -893,7 +900,7 @@ int read_main_config_file(char *main_config_file){
 
 		else if(!strcmp(variable,"soft_state_dependencies")){
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for soft_state_dependencies");
+				dummy=asprintf(&error_message,"Illegal value for soft_state_dependencies");
 				error=TRUE;
 				break;
 			        }
@@ -913,7 +920,7 @@ int read_main_config_file(char *main_config_file){
 			else if(!strcmp(value,"m"))
 				log_rotation_method=LOG_ROTATION_MONTHLY;
 			else{
-				asprintf(&error_message,"Illegal value for log_rotation_method");
+				dummy=asprintf(&error_message,"Illegal value for log_rotation_method");
 				error=TRUE;
 				break;
 			        }
@@ -922,7 +929,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"log_archive_path")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Log archive path too long");
+				dummy=asprintf(&error_message,"Log archive path too long");
 				error=TRUE;
 				break;
 				}
@@ -960,7 +967,7 @@ int read_main_config_file(char *main_config_file){
 				service_inter_check_delay_method=ICD_USER;
 				scheduling_info.service_inter_check_delay=strtod(value,NULL);
 				if(scheduling_info.service_inter_check_delay<=0.0){
-					asprintf(&error_message,"Illegal value for service_inter_check_delay_method");
+					dummy=asprintf(&error_message,"Illegal value for service_inter_check_delay_method");
 					error=TRUE;
 					break;
 				        }
@@ -971,7 +978,7 @@ int read_main_config_file(char *main_config_file){
 			strip(value);
 			max_service_check_spread=atoi(value);
 			if(max_service_check_spread<1){
-				asprintf(&error_message,"Illegal value for max_service_check_spread");
+				dummy=asprintf(&error_message,"Illegal value for max_service_check_spread");
 				error=TRUE;
 				break;
 			        }
@@ -989,7 +996,7 @@ int read_main_config_file(char *main_config_file){
 				host_inter_check_delay_method=ICD_USER;
 				scheduling_info.host_inter_check_delay=strtod(value,NULL);
 				if(scheduling_info.host_inter_check_delay<=0.0){
-					asprintf(&error_message,"Illegal value for host_inter_check_delay_method");
+					dummy=asprintf(&error_message,"Illegal value for host_inter_check_delay_method");
 					error=TRUE;
 					break;
 				        }
@@ -1000,7 +1007,7 @@ int read_main_config_file(char *main_config_file){
 
 			max_host_check_spread=atoi(value);
 			if(max_host_check_spread<1){
-				asprintf(&error_message,"Illegal value for max_host_check_spread");
+				dummy=asprintf(&error_message,"Illegal value for max_host_check_spread");
 				error=TRUE;
 				break;
 			        }
@@ -1021,7 +1028,7 @@ int read_main_config_file(char *main_config_file){
 
 			max_parallel_service_checks=atoi(value);
 			if(max_parallel_service_checks<0){
-				asprintf(&error_message,"Illegal value for max_concurrent_checks");
+				dummy=asprintf(&error_message,"Illegal value for max_concurrent_checks");
 				error=TRUE;
 				break;
 			        }
@@ -1031,7 +1038,7 @@ int read_main_config_file(char *main_config_file){
 
 			check_reaper_interval=atoi(value);
 			if(check_reaper_interval<1){
-				asprintf(&error_message,"Illegal value for check_result_reaper_frequency");
+				dummy=asprintf(&error_message,"Illegal value for check_result_reaper_frequency");
 				error=TRUE;
 				break;
 			        }
@@ -1041,7 +1048,7 @@ int read_main_config_file(char *main_config_file){
 
 			max_check_reaper_time=atoi(value);
 			if(max_check_reaper_time<1){
-				asprintf(&error_message,"Illegal value for max_check_result_reaper_time");
+				dummy=asprintf(&error_message,"Illegal value for max_check_result_reaper_time");
 				error=TRUE;
 				break;
 			        }
@@ -1051,7 +1058,7 @@ int read_main_config_file(char *main_config_file){
 
 			sleep_time=atof(value);
 			if(sleep_time<=0.0){
-				asprintf(&error_message,"Illegal value for sleep_time");
+				dummy=asprintf(&error_message,"Illegal value for sleep_time");
 				error=TRUE;
 				break;
 			        }
@@ -1061,7 +1068,7 @@ int read_main_config_file(char *main_config_file){
 
 			interval_length=atoi(value);
 			if(interval_length<1){
-				asprintf(&error_message,"Illegal value for interval_length");
+				dummy=asprintf(&error_message,"Illegal value for interval_length");
 				error=TRUE;
 				break;
 			        }
@@ -1070,7 +1077,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"check_external_commands")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for check_external_commands");
+				dummy=asprintf(&error_message,"Illegal value for check_external_commands");
 				error=TRUE;
 				break;
 			        }
@@ -1083,7 +1090,7 @@ int read_main_config_file(char *main_config_file){
 			command_check_interval_is_seconds=(strstr(value,"s"))?TRUE:FALSE;
 			command_check_interval=atoi(value);
 			if(command_check_interval<-1 || command_check_interval==0){
-				asprintf(&error_message,"Illegal value for command_check_interval");
+				dummy=asprintf(&error_message,"Illegal value for command_check_interval");
 				error=TRUE;
 				break;
 			        }
@@ -1092,7 +1099,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"check_for_orphaned_services")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for check_for_orphaned_services");
+				dummy=asprintf(&error_message,"Illegal value for check_for_orphaned_services");
 				error=TRUE;
 				break;
 			        }
@@ -1103,7 +1110,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"check_for_orphaned_hosts")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for check_for_orphaned_hosts");
+				dummy=asprintf(&error_message,"Illegal value for check_for_orphaned_hosts");
 				error=TRUE;
 				break;
 			        }
@@ -1114,7 +1121,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"check_service_freshness")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for check_service_freshness");
+				dummy=asprintf(&error_message,"Illegal value for check_service_freshness");
 				error=TRUE;
 				break;
 			        }
@@ -1125,7 +1132,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"check_host_freshness")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for check_host_freshness");
+				dummy=asprintf(&error_message,"Illegal value for check_host_freshness");
 				error=TRUE;
 				break;
 			        }
@@ -1137,7 +1144,7 @@ int read_main_config_file(char *main_config_file){
 
 			service_freshness_check_interval=atoi(value);
 			if(service_freshness_check_interval<=0){
-				asprintf(&error_message,"Illegal value for service_freshness_check_interval");
+				dummy=asprintf(&error_message,"Illegal value for service_freshness_check_interval");
 				error=TRUE;
 				break;
 			        }
@@ -1147,7 +1154,7 @@ int read_main_config_file(char *main_config_file){
 
 			host_freshness_check_interval=atoi(value);
 			if(host_freshness_check_interval<=0){
-				asprintf(&error_message,"Illegal value for host_freshness_check_interval");
+				dummy=asprintf(&error_message,"Illegal value for host_freshness_check_interval");
 				error=TRUE;
 				break;
 			        }
@@ -1155,7 +1162,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"auto_reschedule_checks")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for auto_reschedule_checks");
+				dummy=asprintf(&error_message,"Illegal value for auto_reschedule_checks");
 				error=TRUE;
 				break;
 			        }
@@ -1167,7 +1174,7 @@ int read_main_config_file(char *main_config_file){
 
 			auto_rescheduling_interval=atoi(value);
 			if(auto_rescheduling_interval<=0){
-				asprintf(&error_message,"Illegal value for auto_rescheduling_interval");
+				dummy=asprintf(&error_message,"Illegal value for auto_rescheduling_interval");
 				error=TRUE;
 				break;
 			        }
@@ -1177,7 +1184,7 @@ int read_main_config_file(char *main_config_file){
 
 			auto_rescheduling_window=atoi(value);
 			if(auto_rescheduling_window<=0){
-				asprintf(&error_message,"Illegal value for auto_rescheduling_window");
+				dummy=asprintf(&error_message,"Illegal value for auto_rescheduling_window");
 				error=TRUE;
 				break;
 			        }
@@ -1185,9 +1192,6 @@ int read_main_config_file(char *main_config_file){
 
 		else if(!strcmp(variable,"aggregate_status_updates")){
 
-			/* DEPRECATED - ALL UPDATED ARE AGGREGATED AS OF Icinga 3.X */
-			/*aggregate_status_updates=(atoi(value)>0)?TRUE:FALSE;*/
-
 			logit(NSLOG_CONFIG_WARNING,TRUE,"Warning: aggregate_status_updates directive ignored.  All status file updates are now aggregated.");
 		        }
 
@@ -1195,7 +1199,7 @@ int read_main_config_file(char *main_config_file){
 
 			status_update_interval=atoi(value);
 			if(status_update_interval<=1){
-				asprintf(&error_message,"Illegal value for status_update_interval");
+				dummy=asprintf(&error_message,"Illegal value for status_update_interval");
 				error=TRUE;
 				break;
 			        }
@@ -1206,7 +1210,7 @@ int read_main_config_file(char *main_config_file){
 			time_change_threshold=atoi(value);
 
 			if(time_change_threshold<=5){
-				asprintf(&error_message,"Illegal value for time_change_threshold");
+				dummy=asprintf(&error_message,"Illegal value for time_change_threshold");
 				error=TRUE;
 				break;
 			        }
@@ -1225,7 +1229,7 @@ int read_main_config_file(char *main_config_file){
 
 			low_service_flap_threshold=strtod(value,NULL);
 			if(low_service_flap_threshold<=0.0 || low_service_flap_threshold>=100.0){
-				asprintf(&error_message,"Illegal value for low_service_flap_threshold");
+				dummy=asprintf(&error_message,"Illegal value for low_service_flap_threshold");
 				error=TRUE;
 				break;
 			        }
@@ -1235,7 +1239,7 @@ int read_main_config_file(char *main_config_file){
 
 			high_service_flap_threshold=strtod(value,NULL);
 			if(high_service_flap_threshold<=0.0 ||  high_service_flap_threshold>100.0){
-				asprintf(&error_message,"Illegal value for high_service_flap_threshold");
+				dummy=asprintf(&error_message,"Illegal value for high_service_flap_threshold");
 				error=TRUE;
 				break;
 			        }
@@ -1245,7 +1249,7 @@ int read_main_config_file(char *main_config_file){
 
 			low_host_flap_threshold=strtod(value,NULL);
 			if(low_host_flap_threshold<=0.0 || low_host_flap_threshold>=100.0){
-				asprintf(&error_message,"Illegal value for low_host_flap_threshold");
+				dummy=asprintf(&error_message,"Illegal value for low_host_flap_threshold");
 				error=TRUE;
 				break;
 			        }
@@ -1255,7 +1259,7 @@ int read_main_config_file(char *main_config_file){
 
 			high_host_flap_threshold=strtod(value,NULL);
 			if(high_host_flap_threshold<=0.0 || high_host_flap_threshold>100.0){
-				asprintf(&error_message,"Illegal value for high_host_flap_threshold");
+				dummy=asprintf(&error_message,"Illegal value for high_host_flap_threshold");
 				error=TRUE;
 				break;
 			        }
@@ -1281,7 +1285,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"p1_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"P1 file is too long");
+				dummy=asprintf(&error_message,"P1 file is too long");
 				error=TRUE;
 				break;
 				}
@@ -1322,7 +1326,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"daemon_dumps_core")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for daemon_dumps_core");
+				dummy=asprintf(&error_message,"Illegal value for daemon_dumps_core");
 				error=TRUE;
 				break;
 				}
@@ -1333,7 +1337,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_large_installation_tweaks")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_large_installation_tweaks ");
+				dummy=asprintf(&error_message,"Illegal value for use_large_installation_tweaks ");
 				error=TRUE;
 				break;
 			        }
@@ -1353,7 +1357,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"enable_embedded_perl")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for enable_embedded_perl");
+				dummy=asprintf(&error_message,"Illegal value for enable_embedded_perl");
 				error=TRUE;
 				break;
 			        }
@@ -1364,7 +1368,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"use_embedded_perl_implicitly")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"Illegal value for use_embedded_perl_implicitly");
+				dummy=asprintf(&error_message,"Illegal value for use_embedded_perl_implicitly");
 				error=TRUE;
 				break;
 			        }
@@ -1375,7 +1379,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"stalking_event_handlers_for_hosts")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"stalking_event_handlers_for_hosts");
+				dummy=asprintf(&error_message,"stalking_event_handlers_for_hosts");
 				error=TRUE;
 				break;
 			        }
@@ -1386,7 +1390,7 @@ int read_main_config_file(char *main_config_file){
 		else if(!strcmp(variable,"stalking_event_handlers_for_services")){
 
 			if(strlen(value)!=1||value[0]<'0'||value[0]>'1'){
-				asprintf(&error_message,"stalking_event_handlers_for_services");
+				dummy=asprintf(&error_message,"stalking_event_handlers_for_services");
 				error=TRUE;
 				break;
 			        }
@@ -1413,13 +1417,15 @@ int read_main_config_file(char *main_config_file){
 			event_profiling_enabled=(atoi(value)>0)?TRUE:FALSE;
 #endif
 			}
-
+		else if(!strcmp(variable,"allow_empty_hostgroup_assignment")){
+			allow_empty_hostgroup_assignment=(atoi(value)>0)?TRUE:FALSE;
+			}
 
 		/*** AUTH_FILE VARIABLE USED BY EMBEDDED PERL INTERPRETER ***/
 		else if(!strcmp(variable,"auth_file")){
 
 			if(strlen(value)>MAX_FILENAME_LENGTH-1){
-				asprintf(&error_message,"Auth file is too long");
+				dummy=asprintf(&error_message,"Auth file is too long");
 				error=TRUE;
 				break;
 			        }
@@ -1462,7 +1468,7 @@ int read_main_config_file(char *main_config_file){
 
 		/* we don't know what this variable is... */
 		else{
-			asprintf(&error_message,"UNKNOWN VARIABLE");
+			dummy=asprintf(&error_message,"UNKNOWN VARIABLE");
 			error=TRUE;
 			break;
 			}
@@ -1739,7 +1745,7 @@ int pre_flight_check(void){
 		printf("Checking misc settings...\n");
 
 	/* check if we can write to temp_path */
-       	asprintf(&buf,"%s/nagiosXXXXXX",temp_path);
+       	dummy=asprintf(&buf,"%s/nagiosXXXXXX",temp_path);
        	if((temp_path_fd=mkstemp(buf))==-1){
 		logit(NSLOG_VERIFICATION_ERROR,TRUE,"\tError: Unable to write to temp_path ('%s') - %s\n",temp_path,strerror(errno));
 		errors++;
@@ -1751,7 +1757,7 @@ int pre_flight_check(void){
 	my_free(buf);
 
 	/* check if we can write to check_result_path */
-       	asprintf(&buf,"%s/nagiosXXXXXX",check_result_path);
+       	dummy=asprintf(&buf,"%s/nagiosXXXXXX",check_result_path);
        	if((temp_path_fd=mkstemp(buf))==-1){
 		logit(NSLOG_VERIFICATION_WARNING,TRUE,"\tError: Unable to write to check_result_path ('%s') - %s\n",check_result_path,strerror(errno));
 		errors++;
@@ -1839,6 +1845,7 @@ int pre_flight_object_check(int *w, int *e){
 	servicedependency *temp_sd=NULL;
 	hostdependency *temp_hd=NULL;
 	escalation_condition *temp_escalation_condition=NULL;
+	module *temp_module=NULL;
 	char *buf=NULL;
 	char *temp_command_name="";
 	int found=FALSE;
@@ -2768,6 +2775,26 @@ int pre_flight_object_check(int *w, int *e){
 		printf("\tChecked %d time periods.\n",total_objects);
 
 
+        /*****************************************/
+        /* check all modules...                  */
+        /*****************************************/
+        if(verify_config==TRUE)
+                printf("Checking modules...\n");
+
+        for(temp_module=module_list,total_objects=0;temp_module!=NULL;temp_module=temp_module->next,total_objects++){
+
+                /* check for illegal characters in module name */
+                if(use_precached_objects==FALSE){
+                        if(contains_illegal_object_chars(temp_module->name)==TRUE){
+                                logit(NSLOG_VERIFICATION_ERROR,TRUE,"Error: The name of module '%s' contains one or more illegal characters.",temp_module->name);
+                                errors++;
+                                }
+                        }
+                }
+
+        if(verify_config==TRUE)
+                printf("\tChecked %d modules.\n",total_objects);
+
 
        /* update warning and error count */
 	*w+=warnings;
