@@ -3,7 +3,8 @@
  * LOGGING.C - Log file functions for use with Icinga
  *
  * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
- * Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org)
+ * Copyright (c) 2009-2011 Nagios Core Development Team and Community Contributors
+ * Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org)
  *
  * License:
  *
@@ -34,8 +35,6 @@ extern char	*log_file;
 extern char     *temp_file;
 extern char	*log_archive_path;
 
-extern char     *macro_x[MACRO_X_COUNT];
-
 extern host     *host_list;
 extern service  *service_list;
 
@@ -65,7 +64,37 @@ extern int      debug_verbosity;
 extern unsigned long max_debug_file_size;
 FILE            *debug_file_fp=NULL;
 
+int dummy;	/* reduce compiler warnings */
 
+static pthread_mutex_t debug_fp_lock;
+
+/*
+ * since we don't want child processes to hang indefinitely
+ * in case they inherit a locked lock, we use soft-locking
+ * here, which basically tries to acquire the lock for a
+ * short while and then gives up, returning -1 to signal
+ * the error
+ */
+static inline int soft_lock(pthread_mutex_t *lock){
+	int i;
+
+	for (i = 0; i < 5; i++) {
+		if (!pthread_mutex_trylock(lock)) {
+			/* success */
+			return 0;
+		}
+
+		if (errno == EDEADLK) {
+			/* we already have the lock */
+			return 0;
+		}
+
+		/* sleep briefly */
+		usleep(30);
+	}
+
+	return -1; /* we failed to get the lock. Nothing to do */
+}
 
 
 /******************************************************************/
@@ -258,6 +287,7 @@ int log_service_event(service *svc){
 	char *processed_buffer=NULL;
 	unsigned long log_options=0L;
 	host *temp_host=NULL;
+	icinga_macros mac;
 
 	/* don't log soft errors if the user doesn't want to */
 	if(svc->state_type==SOFT_STATE && !log_service_retries)
@@ -278,18 +308,21 @@ int log_service_event(service *svc){
 		return ERROR;
 
 	/* grab service macros */
-	clear_volatile_macros();
-	grab_host_macros(temp_host);
-	grab_service_macros(svc);
+	memset(&mac, 0, sizeof(mac));
+	grab_host_macros_r(&mac, temp_host);
+	grab_service_macros_r(&mac, svc);
 
+	/* XXX: replace the macro madness with some simple helpers instead */
 	/* either log only the output, or if enabled, add long_output */
 	if(log_long_plugin_output==TRUE && svc->long_plugin_output!=NULL) {
-		asprintf(&temp_buffer,"SERVICE ALERT: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\\n%s\n",svc->host_name,svc->description,(svc->plugin_output==NULL)?"":svc->plugin_output,svc->long_plugin_output);
+		dummy=asprintf(&temp_buffer,"SERVICE ALERT: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\\n%s\n",svc->host_name,svc->description,(svc->plugin_output==NULL)?"":svc->plugin_output,svc->long_plugin_output);
 	} else {
-		asprintf(&temp_buffer,"SERVICE ALERT: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\n",svc->host_name,svc->description,(svc->plugin_output==NULL)?"":svc->plugin_output);
+		dummy=asprintf(&temp_buffer,"SERVICE ALERT: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\n",svc->host_name,svc->description,(svc->plugin_output==NULL)?"":svc->plugin_output);
 	}
 
-	process_macros(temp_buffer,&processed_buffer,0);
+	process_macros_r(&mac, temp_buffer,&processed_buffer,0);
+	clear_host_macros_r(&mac);
+	clear_service_macros_r(&mac);
 
 	write_to_all_logs(processed_buffer,log_options);
 
@@ -297,7 +330,7 @@ int log_service_event(service *svc){
 	my_free(processed_buffer);
 
 	return OK;
-	}
+}
 
 
 /* write a host problem/recovery to the log file */
@@ -305,10 +338,11 @@ int log_host_event(host *hst){
 	char *temp_buffer=NULL;
 	char *processed_buffer=NULL;
 	unsigned long log_options=0L;
+	icinga_macros mac;
 
 	/* grab the host macros */
-	clear_volatile_macros();
-	grab_host_macros(hst);
+	memset(&mac, 0, sizeof(mac));
+	grab_host_macros_r(&mac, hst);
 
 	/* get the log options */
 	if(hst->current_state==HOST_DOWN)
@@ -318,52 +352,57 @@ int log_host_event(host *hst){
 	else
 		log_options=NSLOG_HOST_UP;
 
-
+	/* XXX: replace the macro madness with some simple helpers instead */
 	/* either log only the output, or if enabled, add long_output */
 	if(log_long_plugin_output==TRUE && hst->long_plugin_output!=NULL) {
-		asprintf(&temp_buffer,"HOST ALERT: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\\n%s\n",hst->name,(hst->plugin_output==NULL)?"":hst->plugin_output,hst->long_plugin_output);
+		dummy=asprintf(&temp_buffer,"HOST ALERT: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\\n%s\n",hst->name,(hst->plugin_output==NULL)?"":hst->plugin_output,hst->long_plugin_output);
 	} else {
-		asprintf(&temp_buffer,"HOST ALERT: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\n",hst->name,(hst->plugin_output==NULL)?"":hst->plugin_output);
+		dummy=asprintf(&temp_buffer,"HOST ALERT: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\n",hst->name,(hst->plugin_output==NULL)?"":hst->plugin_output);
 	}
 
-	process_macros(temp_buffer,&processed_buffer,0);
+	process_macros_r(&mac, temp_buffer,&processed_buffer,0);
 
 	write_to_all_logs(processed_buffer,log_options);
 
+	clear_host_macros_r(&mac);
 	my_free(temp_buffer);
 	my_free(processed_buffer);
 
 	return OK;
-        }
+}
 
 
 /* logs host states */
 int log_host_states(int type, time_t *timestamp){
 	char *temp_buffer=NULL;
 	char *processed_buffer=NULL;
-	host *temp_host=NULL;;
+	host *temp_host=NULL;
+	icinga_macros mac;
 
 	/* bail if we shouldn't be logging initial states */
 	if(type==INITIAL_STATES && log_initial_states==FALSE)
 		return OK;
 
+	memset(&mac, 0, sizeof(mac));
+
 	for(temp_host=host_list;temp_host!=NULL;temp_host=temp_host->next){
 
 		/* grab the host macros */
-		clear_volatile_macros();
-		grab_host_macros(temp_host);
+		grab_host_macros_r(&mac, temp_host);
 
-		asprintf(&temp_buffer,"%s HOST STATE: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_host->name,(temp_host->plugin_output==NULL)?"":temp_host->plugin_output);
-		process_macros(temp_buffer,&processed_buffer,0);
+		dummy=asprintf(&temp_buffer,"%s HOST STATE: %s;$HOSTSTATE$;$HOSTSTATETYPE$;$HOSTATTEMPT$;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_host->name,(temp_host->plugin_output==NULL)?"":temp_host->plugin_output);
+		process_macros_r(&mac, temp_buffer,&processed_buffer,0);
 
 		write_to_all_logs_with_timestamp(processed_buffer,NSLOG_INFO_MESSAGE,timestamp);
 
+		clear_host_macros_r(&mac);
+
 		my_free(temp_buffer);
 		my_free(processed_buffer);
 	        }
 
 	return OK;
-        }
+}
 
 
 /* logs service states */
@@ -371,12 +410,15 @@ int log_service_states(int type, time_t *timestamp){
 	char *temp_buffer=NULL;
 	char *processed_buffer=NULL;
 	service *temp_service=NULL;
-	host *temp_host=NULL;;
+	host *temp_host=NULL;
+	icinga_macros mac;
 
 	/* bail if we shouldn't be logging initial states */
 	if(type==INITIAL_STATES && log_initial_states==FALSE)
 		return OK;
 
+	memset(&mac, 0, sizeof(mac));
+
 	for(temp_service=service_list;temp_service!=NULL;temp_service=temp_service->next){
 
 		/* find the associated host */
@@ -384,15 +426,18 @@ int log_service_states(int type, time_t *timestamp){
 			continue;
 
 		/* grab service macros */
-		clear_volatile_macros();
-		grab_host_macros(temp_host);
-		grab_service_macros(temp_service);
+		grab_host_macros_r(&mac, temp_host);
+		grab_service_macros_r(&mac, temp_service);
 
-		asprintf(&temp_buffer,"%s SERVICE STATE: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_service->host_name,temp_service->description,temp_service->plugin_output);
-		process_macros(temp_buffer,&processed_buffer,0);
+		/* XXX: macro madness */
+		dummy=asprintf(&temp_buffer,"%s SERVICE STATE: %s;%s;$SERVICESTATE$;$SERVICESTATETYPE$;$SERVICEATTEMPT$;%s\n",(type==INITIAL_STATES)?"INITIAL":"CURRENT",temp_service->host_name,temp_service->description,temp_service->plugin_output);
+		process_macros_r(&mac, temp_buffer,&processed_buffer,0);
 
 		write_to_all_logs_with_timestamp(processed_buffer,NSLOG_INFO_MESSAGE,timestamp);
 
+		clear_host_macros_r(&mac);
+		clear_service_macros_r(&mac);
+
 		my_free(temp_buffer);
 		my_free(processed_buffer);
 	        }
@@ -406,7 +451,7 @@ int rotate_log_file(time_t rotation_time){
 	char *temp_buffer=NULL;
 	char method_string[16]="";
 	char *log_archive=NULL;
-	struct tm *t;
+	struct tm *t, tm_s;
 	int rename_result=0;
 	int stat_result=-1;
 	struct stat log_file_stat;
@@ -429,12 +474,12 @@ int rotate_log_file(time_t rotation_time){
 	last_log_rotation=time(NULL);
 	update_program_status(FALSE);
 
-	t=localtime(&rotation_time);
+	t = localtime_r(&rotation_time, &tm_s);
 
 	stat_result = stat(log_file, &log_file_stat);
 
 	/* get the archived filename to use */
-	asprintf(&log_archive,"%s%sicinga-%02d-%02d-%d-%02d.log", log_archive_path, (log_archive_path[strlen(log_archive_path)-1]=='/')?"":"/", t->tm_mon+1, t->tm_mday, t->tm_year+1900, t->tm_hour);
+	dummy=asprintf(&log_archive,"%s%sicinga-%02d-%02d-%d-%02d.log", log_archive_path, (log_archive_path[strlen(log_archive_path)-1]=='/')?"":"/", t->tm_mon+1, t->tm_mday, t->tm_year+1900, t->tm_hour);
 
 	/* rotate the log file */
 	rename_result=my_rename(log_file,log_archive);
@@ -445,7 +490,7 @@ int rotate_log_file(time_t rotation_time){
 	        }
 
 	/* record the log rotation after it has been done... */
-	asprintf(&temp_buffer,"LOG ROTATION: %s\n",method_string);
+	dummy=asprintf(&temp_buffer,"LOG ROTATION: %s\n",method_string);
 	write_to_all_logs_with_timestamp(temp_buffer,NSLOG_PROCESS_INFO,&rotation_time);
 	my_free(temp_buffer);
 
@@ -454,7 +499,7 @@ int rotate_log_file(time_t rotation_time){
 
 	if(stat_result==0){
 		chmod(log_file, log_file_stat.st_mode);
-		chown(log_file, log_file_stat.st_uid, log_file_stat.st_gid);
+		dummy=chown(log_file, log_file_stat.st_uid, log_file_stat.st_gid);
 		}
 
 	/* log current host and service state if activated*/
@@ -475,7 +520,7 @@ int write_log_file_info(time_t *timestamp){
 	char *temp_buffer=NULL;
 
 	/* write log version */
-	asprintf(&temp_buffer,"LOG VERSION: %s\n",LOG_VERSION_2);
+	dummy=asprintf(&temp_buffer,"LOG VERSION: %s\n",LOG_VERSION_2);
 	write_to_all_logs_with_timestamp(temp_buffer,NSLOG_PROCESS_INFO,timestamp);
 	my_free(temp_buffer);
 
@@ -528,6 +573,15 @@ int log_debug_info(int level, int verbosity, const char *fmt, ...){
 	if(debug_file_fp==NULL)
 		return ERROR;
 
+	/*
+	 * lock it so concurrent threads don't stomp on each other's
+	 * writings. We maintain the lock until we've (optionally)
+	 * renamed the file.
+	 * If soft_lock() fails we return early.
+	 */
+	if (soft_lock(&debug_fp_lock) < 0)
+		return ERROR;
+
 	/* write the timestamp */
 	gettimeofday(&current_time,NULL);
 	fprintf(debug_file_fp,"[%lu.%06lu] [%03d.%d] [pid=%lu] ",current_time.tv_sec,current_time.tv_usec,level,verbosity,(unsigned long)getpid());
@@ -547,7 +601,7 @@ int log_debug_info(int level, int verbosity, const char *fmt, ...){
 		close_debug_log();
 		
 		/* rotate the log file */
-		asprintf(&temp_path,"%s.old",debug_file);
+		dummy=asprintf(&temp_path,"%s.old",debug_file);
 		if(temp_path){
 
 			/* unlink the old debug file */
@@ -564,6 +618,8 @@ int log_debug_info(int level, int verbosity, const char *fmt, ...){
 		open_debug_log();
 		}
 
+	pthread_mutex_unlock(&debug_fp_lock);
+
 	return OK;
-	}
+}
 
