@@ -3,7 +3,7 @@
  * OUTAGES.C -  Icinga Network Outages CGI
  *
  * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
- * Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org)
+ * Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org)
  *
  * License:
  *
@@ -46,6 +46,17 @@ extern char url_images_path[MAX_FILENAME_LENGTH];
 extern char url_logo_images_path[MAX_FILENAME_LENGTH];
 extern char log_file[MAX_FILENAME_LENGTH];
 
+int display_type=DISPLAY_HOSTS;
+int show_all_hosts=TRUE;
+int show_all_hostgroups=TRUE;
+int show_all_servicegroups=TRUE;
+
+char *host_name=NULL;
+char *host_filter=NULL;
+char *hostgroup_name=NULL;
+char *servicegroup_name=NULL;
+char *service_desc=NULL;
+char *service_filter=NULL;
 
 /* HOSTOUTAGE structure */
 typedef struct hostoutage_struct{
@@ -116,7 +127,7 @@ int main(void){
 	result=read_cgi_config_file(get_cgi_config_location());
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
-		cgi_config_file_error(get_cgi_config_location());
+		print_error(get_cgi_config_location(), ERROR_CGI_CFG_FILE);
 		document_footer(CGI_ID);
 		return ERROR;
 	        }
@@ -125,7 +136,7 @@ int main(void){
 	result=read_main_config_file(main_config_file);
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
-		main_config_file_error(main_config_file);
+		print_error(main_config_file, ERROR_CGI_MAIN_CFG);
 		document_footer(CGI_ID);
 		return ERROR;
 	        }
@@ -134,7 +145,7 @@ int main(void){
 	result=read_all_object_configuration_data(main_config_file,READ_ALL_OBJECT_DATA);
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
-		object_data_error();
+		print_error(NULL, ERROR_CGI_OBJECT_DATA);
 		document_footer(CGI_ID);
 		return ERROR;
                 }
@@ -143,7 +154,7 @@ int main(void){
 	result=read_all_status_data(get_cgi_config_location(),READ_ALL_STATUS_DATA);
 	if(result==ERROR && daemon_check==TRUE){
 		document_header(CGI_ID,FALSE);
-		status_data_error();
+		print_error(NULL, ERROR_CGI_STATUS_DATA);
 		document_footer(CGI_ID);
 		free_memory();
 		return ERROR;
@@ -230,7 +241,12 @@ int process_cgivars(void){
 		else if(!strcmp(variables[x],"csvoutput")){
 			display_header=FALSE;
 			content_type=CSV_CONTENT;
-			}
+		}
+
+		else if(!strcmp(variables[x],"jsonoutput")){
+			display_header=FALSE;
+			content_type=JSON_CONTENT;
+		}
 
 			/* we found the embed option */
 		else if(!strcmp(variables[x],"embedded"))
@@ -276,16 +292,7 @@ void display_network_outages(void){
 	time_t current_time;
 	char state_duration[48];
 	int total_entries=0;
-
-	/* user must be authorized for all hosts.. */
-	if(is_authorized_for_all_hosts(&current_authdata)==FALSE){
-
-		printf("<P><DIV CLASS='errorMessage'>It appears as though you do not have permission to view information you requested...</DIV></P>\n");
-		printf("<P><DIV CLASS='errorDescription'>If you believe this is an error, check the HTTP server authentication requirements for accessing this CGI<br>");
-		printf("and check the authorization options in your CGI configuration file.</DIV></P>\n");
-
-		return;
-		}
+	int json_start=TRUE;
 
 	/* find all hosts that are causing network outages */
 	find_hosts_causing_outages();
@@ -303,7 +310,9 @@ void display_network_outages(void){
 			number_of_blocking_problem_hosts++;
 		}
 
-	if(content_type==CSV_CONTENT) {
+	if(content_type==JSON_CONTENT) {
+		printf("\"outages\": [\n");
+	} else if(content_type==CSV_CONTENT) {
 		printf("%sSEVERITY%s%s",csv_data_enclosure,csv_data_enclosure,csv_delimiter);
 		printf("%sHOST%s%s",csv_data_enclosure,csv_data_enclosure,csv_delimiter);
 		printf("%sSTATE%s%s",csv_data_enclosure,csv_data_enclosure,csv_delimiter);
@@ -319,11 +328,7 @@ void display_network_outages(void){
 		printf("<TABLE BORDER=0 CLASS='data'>\n");
 
                 /* add export to csv link */
-                if(getenv("QUERY_STRING")!=NULL) {
-			printf("<TR><TD colspan='8'><DIV class='csv_export_link'><A HREF='%s?%s&csvoutput' target='_blank'>Export to CSV</A></DIV></TD></TR>\n",OUTAGES_CGI,strdup(getenv("QUERY_STRING")));
-		} else {
-			printf("<TR><TD colspan='8'><DIV class='csv_export_link'><A HREF='%s?csvoutput' target='_blank'>Export to CSV</A></DIV></TD></TR>\n",OUTAGES_CGI);
-		}
+		printf("<TR><TD colspan='8'><DIV class='csv_export_link'><A HREF='%s' target='_blank'>Export to CSV</A></DIV></TD></TR>\n",get_export_csv_link(OUTAGES_CGI));
 
 		printf("<TR>\n");
 		printf("<TH CLASS='data'>Severity</TH><TH CLASS='data'>Host</TH><TH CLASS='data'>State</TH><TH CLASS='data'>Notes</TH><TH CLASS='data'>State Duration</TH><TH CLASS='data'># Hosts Affected</TH><TH CLASS='data'># Services Affected</TH><TH CLASS='data'>Actions</TH>\n");
@@ -363,7 +368,15 @@ void display_network_outages(void){
 		else if(temp_hoststatus->status==HOST_DOWN)
 			status="DOWN";
 
-		if(content_type==CSV_CONTENT) {
+		if(content_type==JSON_CONTENT) {
+			// always add a comma, except for the first line
+			if (json_start==FALSE)
+				printf(",\n");
+			json_start=FALSE;
+			printf("{ \"severity\": %d, ",temp_hostoutage->severity);
+			printf(" \"host\": \"%s\", ",(temp_hostoutage->hst->display_name!=NULL)?json_encode(temp_hostoutage->hst->display_name):json_encode(temp_hostoutage->hst->name));
+			printf(" \"state\": \"%s\", ",status);
+		} else if(content_type==CSV_CONTENT) {
 			printf("%s%d%s%s",csv_data_enclosure,temp_hostoutage->severity,csv_data_enclosure,csv_delimiter);
 			printf("%s%s%s%s",csv_data_enclosure,(temp_hostoutage->hst->display_name!=NULL)?temp_hostoutage->hst->display_name:temp_hostoutage->hst->name,csv_data_enclosure,csv_delimiter);
 			printf("%s%s%s%s",csv_data_enclosure,status,csv_data_enclosure,csv_delimiter);
@@ -376,7 +389,9 @@ void display_network_outages(void){
 		}
 
 		total_comments=number_of_host_comments(temp_hostoutage->hst->name);
-		if(content_type==CSV_CONTENT) {
+		if(content_type==JSON_CONTENT) {
+			printf(" \"notes\": %d, ",total_comments);
+		} else if(content_type==CSV_CONTENT) {
 			printf("%s%d%s%s",csv_data_enclosure,total_comments,csv_data_enclosure,csv_delimiter);
 		} else {
 			if(total_comments>0){
@@ -397,7 +412,11 @@ void display_network_outages(void){
 		snprintf(state_duration,sizeof(state_duration)-1,"%2dd %2dh %2dm %2ds%s",days,hours,minutes,seconds,(temp_hoststatus->last_state_change==(time_t)0)?"+":"");
 		state_duration[sizeof(state_duration)-1]='\x0';
 
-		if(content_type==CSV_CONTENT) {
+		if(content_type==JSON_CONTENT) {
+			printf(" \"state_duration\": \"%s\", ",state_duration);
+			printf(" \"hosts_affected\": %d, ",temp_hostoutage->affected_child_hosts);
+			printf(" \"services_affected\": %d }\n",temp_hostoutage->affected_child_services);
+		} else if(content_type==CSV_CONTENT) {
 			printf("%s%s%s%s",csv_data_enclosure,state_duration,csv_data_enclosure,csv_delimiter);
 			printf("%s%d%s%s",csv_data_enclosure,temp_hostoutage->affected_child_hosts,csv_data_enclosure,csv_delimiter);
 			printf("%s%d%s\n",csv_data_enclosure,temp_hostoutage->affected_child_services,csv_data_enclosure);
@@ -425,13 +444,15 @@ void display_network_outages(void){
 		}
 	}
 
-	if(content_type!=CSV_CONTENT) {
+	if(content_type!=CSV_CONTENT && content_type!=JSON_CONTENT) {
 		printf("</TABLE>\n");
 
 		printf("</DIV></P>\n");
 
 		if(total_entries==0)
 			printf("<DIV CLASS='itemTotalsTitle'>%d Blocking Outages Displayed</DIV>\n",total_entries);
+	}else if (content_type==JSON_CONTENT){
+		printf("\n]\n");
 	}
 
 	/* free memory allocated to the host outage list */
@@ -458,6 +479,9 @@ void find_hosts_causing_outages(void){
 			if(temp_host==NULL)
 				continue;
 
+			if (is_authorized_for_host(temp_host,&current_authdata)==FALSE)
+				continue;
+
 			/* if the route to this host is not blocked, it is a causing an outage */
 			if(is_route_to_host_blocked(temp_host)==FALSE)
 				add_hostoutage(temp_host);
