@@ -3,7 +3,7 @@
  * STATUSMAP.C - Icinga Network Status Map CGI
  *
  * Copyright (c) 1999-2008 Ethan Galstad (egalstad@nagios.org)
- * Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org)
+ * Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org)
  *
  * Description:
  *
@@ -39,6 +39,8 @@
 #include <gd.h>			/* Boutell's GD library function */
 #include <gdfonts.h>		/* GD library small font definition */
 
+static icinga_macros *mac;
+
 /*#define DEBUG*/
 
 #define UNKNOWN_GD2_ICON      "unknown.gd2"
@@ -166,9 +168,6 @@ extern int color_transparency_index_r;
 extern int color_transparency_index_g;
 extern int color_transparency_index_b;
 
-int show_all_hosts=TRUE;
-char *host_name="all";
-
 extern int embedded;
 extern int refresh;
 extern int display_header;
@@ -221,11 +220,25 @@ layer *layer_list=NULL;
 int exclude_layers=TRUE;
 int all_layers=FALSE;
 
+int display_type=DISPLAY_HOSTS;
+int show_all_hosts=TRUE;
+int show_all_hostgroups=TRUE;
+int show_all_servicegroups=TRUE;
+
+char *host_name="all";
+char *host_filter=NULL;
+char *hostgroup_name=NULL;
+char *servicegroup_name=NULL;
+char *service_desc=NULL;
+char *service_filter=NULL;
+
 int CGI_ID=STATUSMAP_CGI_ID;
 
 int main(int argc, char **argv){
 	int result;
 
+	mac = get_global_macros();
+
 	/* reset internal variables */
 	reset_cgi_vars();
 
@@ -234,7 +247,7 @@ int main(int argc, char **argv){
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
 		if(content_type==HTML_CONTENT)
-			cgi_config_file_error(get_cgi_config_location());
+			print_error(get_cgi_config_location(), ERROR_CGI_CFG_FILE);
 		document_footer(CGI_ID);
 		return ERROR;
 	        }
@@ -250,7 +263,7 @@ int main(int argc, char **argv){
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
 		if(content_type==HTML_CONTENT)
-			main_config_file_error(main_config_file);
+			print_error(main_config_file, ERROR_CGI_MAIN_CFG);
 		document_footer(CGI_ID);
 		return ERROR;
 	        }
@@ -260,7 +273,7 @@ int main(int argc, char **argv){
 	if(result==ERROR){
 		document_header(CGI_ID,FALSE);
 		if(content_type==HTML_CONTENT)
-			object_data_error();
+			print_error(NULL, ERROR_CGI_OBJECT_DATA);
 		document_footer(CGI_ID);
 		return ERROR;
                 }
@@ -270,7 +283,7 @@ int main(int argc, char **argv){
 	if(result==ERROR && daemon_check==TRUE){
 		document_header(CGI_ID,FALSE);
 		if(content_type==HTML_CONTENT)
-			status_data_error();
+			print_error(NULL, ERROR_CGI_STATUS_DATA);
 		document_footer(CGI_ID);
 		free_memory();
 		return ERROR;
@@ -1844,7 +1857,7 @@ void write_host_popup_text(host *hst){
 	        }
 
 	/* grab macros */
-	grab_host_macros(hst);
+	grab_host_macros_r(mac, hst);
 
 	/* strip nasty stuff from plugin output */
 	sanitize_plugin_output(temp_status->plugin_output);
@@ -1855,7 +1868,7 @@ void write_host_popup_text(host *hst){
 	if(hst->icon_image==NULL)
 		printf("%s",UNKNOWN_ICON_IMAGE);
 	else{
-		process_macros(hst->icon_image,&processed_string,0);
+		process_macros_r(mac, hst->icon_image,&processed_string,0);
 		printf("%s",processed_string);
 		free(processed_string);
 		}
@@ -1865,6 +1878,7 @@ void write_host_popup_text(host *hst){
 	printf("<tr><td class=\\\"popupText\\\">Name:</td><td class=\\\"popupText\\\"><b>%s</b></td></tr>",escape_string(hst->name));
 	printf("<tr><td class=\\\"popupText\\\">Alias:</td><td class=\\\"popupText\\\"><b>%s</b></td></tr>",escape_string(hst->alias));
 	printf("<tr><td class=\\\"popupText\\\">Address:</td><td class=\\\"popupText\\\"><b>%s</b></td></tr>",html_encode(hst->address,TRUE));
+	printf("<tr><td class=\\\"popupText\\\">Address6:</td><td class=\\\"popupText\\\"><b>%s</b></td></tr>",html_encode(hst->address6,TRUE));
 	printf("<tr><td class=\\\"popupText\\\">State:</td><td class=\\\"popupText\\\"><b>");
 
 	/* get the status of the host (pending, up, down, or unreachable) */
@@ -2230,7 +2244,7 @@ void print_layer_url(int get_method){
 
 	for(temp_layer=layer_list;temp_layer!=NULL;temp_layer=temp_layer->next){
 		if(get_method==TRUE)
-			printf("&layer=%s",temp_layer->layer_name);
+			printf("&layer=%s",escape_string(temp_layer->layer_name));
 		else
 			printf("<input type='hidden' name='layer' value='%s'>\n",escape_string(temp_layer->layer_name));
 	        }
@@ -2625,48 +2639,56 @@ void draw_circular_layer_markup(host *parent, double start_angle, double useable
 			x_offset=nagios_icon_x+(DEFAULT_NODE_WIDTH/2)-canvas_x;
 			y_offset=nagios_icon_y+(DEFAULT_NODE_HEIGHT/2)-canvas_y;
 
-			/* draw "slice" dividers */
-			if(immediate_children>1 || layer>1){
+			/* if the host should be drawn */
+			/* this enforces the privacy of hosts that are not eligble to be drawn */
+			if(temp_host->should_be_drawn==TRUE){
 
-				/* draw "leftmost" divider */
-				gdImageLine(map_image,(int)x_coord[0]+x_offset,(int)y_coord[0]+y_offset,(int)x_coord[1]+x_offset,(int)y_coord[1]+y_offset,color_lightgrey);
+				/* draw "slice" dividers */
+				if(immediate_children>1 || layer>1){
 
-				/* draw "rightmost" divider */
-				gdImageLine(map_image,(int)x_coord[2]+x_offset,(int)y_coord[2]+y_offset,(int)x_coord[3]+x_offset,(int)y_coord[3]+y_offset,color_lightgrey);
-			        }
+					/* draw "leftmost" divider */
+					gdImageLine(map_image,(int)x_coord[0]+x_offset,(int)y_coord[0]+y_offset,(int)x_coord[1]+x_offset,(int)y_coord[1]+y_offset,color_lightgrey);
 
-			/* determine arc drawing angles */
-			arc_start_angle=current_drawing_angle-90.0;
-			while(arc_start_angle<0.0)
-				arc_start_angle+=360.0;
-			arc_end_angle=arc_start_angle+available_angle;
-
-			/* draw inner arc */
-			gdImageArc(map_image,x_offset,y_offset,(radius-(CIRCULAR_DRAWING_RADIUS/2))*2,(radius-(CIRCULAR_DRAWING_RADIUS/2))*2,floor(arc_start_angle),ceil(arc_end_angle),color_lightgrey);
-
-			/* draw outer arc */
-			gdImageArc(map_image,x_offset,y_offset,(radius+(CIRCULAR_DRAWING_RADIUS/2))*2,(radius+(CIRCULAR_DRAWING_RADIUS/2))*2,floor(arc_start_angle),ceil(arc_end_angle),color_lightgrey);
+					/* draw "rightmost" divider */
+					gdImageLine(map_image,(int)x_coord[2]+x_offset,(int)y_coord[2]+y_offset,(int)x_coord[3]+x_offset,(int)y_coord[3]+y_offset,color_lightgrey);
+				        }
 
 
-			/* determine center of "slice" and fill with appropriate color */
-			center_x=-(sin(-(current_drawing_angle+(available_angle/2.0))*(M_PI/180.0))*(radius));
-			center_y=-(sin((90+current_drawing_angle+(available_angle/2.0))*(M_PI/180.0))*(radius));
-			translated_x=center_x+x_offset;
-			translated_y=center_y+y_offset;
+				/* determine arc drawing angles */
+				arc_start_angle=current_drawing_angle-90.0;
+				while(arc_start_angle<0.0)
+					arc_start_angle+=360.0;
+				arc_end_angle=arc_start_angle+available_angle;
 
-			/* determine background color */
-			temp_hoststatus=find_hoststatus(temp_host->name);
-			if(temp_hoststatus==NULL)
-				bgcolor=color_lightgrey;
-			else if(temp_hoststatus->status==HOST_DOWN || temp_hoststatus->status==HOST_UNREACHABLE)
-				bgcolor=color_lightred;
-			else
-				bgcolor=color_lightgreen;
+				/* draw inner arc */
+				gdImageArc(map_image,x_offset,y_offset,(radius-(CIRCULAR_DRAWING_RADIUS/2))*2,(radius-(CIRCULAR_DRAWING_RADIUS/2))*2,floor(arc_start_angle),ceil(arc_end_angle),color_lightgrey);
 
-			/* fill slice with background color */
-			/* the fill function only works with coordinates that are in bounds of the actual image */
-			if(translated_x>0 && translated_y>0 && translated_x<canvas_width && translated_y<canvas_height)
-				gdImageFillToBorder(map_image,translated_x,translated_y,color_lightgrey,bgcolor);
+				/* draw outer arc */
+				gdImageArc(map_image,x_offset,y_offset,(radius+(CIRCULAR_DRAWING_RADIUS/2))*2,(radius+(CIRCULAR_DRAWING_RADIUS/2))*2,floor(arc_start_angle),ceil(arc_end_angle),color_lightgrey);
+
+
+				/* determine center of "slice" and fill with appropriate color */
+				center_x=-(sin(-(current_drawing_angle+(available_angle/2.0))*(M_PI/180.0))*(radius));
+				center_y=-(sin((90+current_drawing_angle+(available_angle/2.0))*(M_PI/180.0))*(radius));
+				translated_x=center_x+x_offset;
+				translated_y=center_y+y_offset;
+
+				/* determine background color */
+				temp_hoststatus=find_hoststatus(temp_host->name);
+				if(temp_hoststatus==NULL)
+					bgcolor=color_lightgrey;
+				else if(temp_hoststatus->status==HOST_DOWN || temp_hoststatus->status==HOST_UNREACHABLE)
+					bgcolor=color_lightred;
+				else
+					bgcolor=color_lightgreen;
+
+
+				/* fill slice with background color */
+				/* the fill function only works with coordinates that are in bounds of the actual image */
+				if(translated_x>0 && translated_y>0 && translated_x<canvas_width && translated_y<canvas_height)
+					gdImageFillToBorder(map_image,translated_x,translated_y,color_lightgrey,bgcolor);
+
+				}
 
 			/* recurse into child host ... */
 			draw_circular_layer_markup(temp_host,current_drawing_angle+((available_angle-clipped_available_angle)/2),clipped_available_angle,layer+1,radius+CIRCULAR_DRAWING_RADIUS);
