@@ -14,6 +14,7 @@ class IcingaApiSearchIdoOci8
 	 * VARIABLES
 	 */
 
+	public $configType = 1;
 	public $tablePrefix = null;
 	public $postProcess = true;
 	public $groupByCols = array();
@@ -65,7 +66,7 @@ class IcingaApiSearchIdoOci8
 			${if_table:cvsh,oh:inner join ${TABLE_PREFIX}customvariablestatus cvsh on oh.id = cvsh.id}
 			${if_table:cvsc,oc,cgm,cg,hcg,h:inner join ${TABLE_PREFIX}customvariablestatus cvsc on oc.id = cvsc.id}
 			where
-				oh.objecttype_id = 1
+				oh.objecttype_id = 1 and h.config_type=${CONFIG_TYPE}
 			${FILTER_AND}
 			${GROUPBY}
 			${ORDERBY}
@@ -85,6 +86,7 @@ class IcingaApiSearchIdoOci8
 			inner join ${TABLE_PREFIX}servicestatus ss on ss.service_object_id = os.id
 			${if_table:ocg,scg,s:inner join ${TABLE_PREFIX}objects ocg on ocg.id = scg.contactgroup_object_id and ocg.objecttype_id = 11}
 			${if_table:hs,s:inner join ${TABLE_PREFIX}hoststatus hs on hs.host_object_id = s.host_object_id}
+			${if_table:h:inner join ${TABLE_PREFIX}hosts h on h.host_object_id = s.host_object_id}
 			${if_table:oh,s:inner join ${TABLE_PREFIX}objects oh on oh.id = s.host_object_id and oh.objecttype_id = 1}
 			${if_table:sgm:left join ${TABLE_PREFIX}servicegroup_members sgm on sgm.service_object_id = os.id}
 			${if_table:sg,sgm:left join ${TABLE_PREFIX}servicegroups sg on sg.id = sgm.servicegroup_id}
@@ -96,7 +98,7 @@ class IcingaApiSearchIdoOci8
 			${if_table:cvss:inner join ${TABLE_PREFIX}customvariablestatus cvss on os.id = cvss.id}
 			${if_table:cvsc,oc,cgm,cg,scg,s:inner join ${TABLE_PREFIX}customvariablestatus cvsc on oc.id = cvsc.id}
 			where
-				os.objecttype_id = 2
+				os.objecttype_id = 2 and s.config_type=${CONFIG_TYPE}
 			${FILTER_AND}
 			${GROUPBY}
 			${ORDERBY}
@@ -194,6 +196,7 @@ class IcingaApiSearchIdoOci8
 				${FIELDS}
 			from
 				${TABLE_PREFIX}logentries le
+				${if_table:i:inner join ${TABLE_PREFIX}instances i on i.id = le.instance_id}
 			 
 			${FILTER}
 			${GROUPBY}
@@ -201,7 +204,7 @@ class IcingaApiSearchIdoOci8
 			${LIMIT}',
 		self::TARGET_HOST_STATUS_SUMMARY => 
 			'select
-				${FIELDS:hs.current_state HOST_STATE, count(hs.current_state) COUNT}
+				${FIELDS:hs.current_state HOST_STATE, count(DISTINCT hs.host_object_id) COUNT}
 			from
 				${TABLE_PREFIX}hoststatus hs
 			inner join ${TABLE_PREFIX}objects oh on oh.id = hs.host_object_id
@@ -219,15 +222,15 @@ class IcingaApiSearchIdoOci8
 			${if_table:ohg,hg,hgm,oh:inner join ${TABLE_PREFIX}objects ohg on ohg.id = hg.hostgroup_object_id}
 			${if_table:cvsh,oh:inner join ${TABLE_PREFIX}customvariablestatus cvsh on oh.id = cvsh.id}
 			${if_table:cvsc,oc,cgm,cg,hcg,h,oh: inner join ${TABLE_PREFIX}customvariablestatus cvsc on oc.id = cvsc.id}
-			
-			${FILTER}
+			where h.config_type=${CONFIG_TYPE}
+			${FILTER_AND}
 			group by
 				hs.current_state
 			${ORDERBY:hs.current_state}
 			${LIMIT}',
 		self::TARGET_SERVICE_STATUS_SUMMARY =>
 			'select
-				${FIELDS:ss.current_state SERVICE_STATE, count(ss.current_state) COUNT}
+				${FIELDS:ss.current_state SERVICE_STATE, count(DISTINCT ss.service_object_id) COUNT}
 			from
 				${TABLE_PREFIX}servicestatus ss
 			inner join ${TABLE_PREFIX}objects os on os.id = ss.service_object_id
@@ -248,8 +251,8 @@ class IcingaApiSearchIdoOci8
 			${if_table:cvss,os:inner join ${TABLE_PREFIX}customvariablestatus cvss on os.id = cvss.id}
 			${if_table:cvsc,oc,cgm,cg,scg,s,os:inner join ${TABLE_PREFIX}customvariablestatus cvsc on oc.id = cvsc.id}
 			left join ${TABLE_PREFIX}objects osg on osg.id = h.host_object_id and osg.is_active = 1
-			
-			${FILTER}
+			where s.config_type=${CONFIG_TYPE}
+			${FILTER_AND}
 			group by
 				ss.current_state
 			${ORDERBY:ss.current_state}
@@ -404,6 +407,64 @@ class IcingaApiSearchIdoOci8
 			${FILTER_AND}
 			${GROUPBY}
 			${ORDERBY}
+			${LIMIT}',
+		
+		self::TARGET_DOWNTIMEHISTORY => 
+			'select
+				distinct ${FIELDS}
+			from
+			${TABLE_PREFIX}downtimehistory dth
+			
+			left join ${TABLE_PREFIX}objects os ON os.id = dth.object_id
+			left join ${TABLE_PREFIX}services s ON s.service_object_id = os.id
+			${if_table:ss:left join ${TABLE_PREFIX}servicestatus ss on ss.service_object_id = os.id}
+			
+			left join ${TABLE_PREFIX}objects oh ON oh.id = s.host_object_id OR oh.id = dth.object_id
+			${if_table:h:inner join ${TABLE_PREFIX}hosts h on h.host_object_id = oh.id}
+			${if_table:hs:inner join ${TABLE_PREFIX}hoststatus hs on hs.host_object_id = ohid}
+			
+			${if_table:hgm:left join ${TABLE_PREFIX}hostgroup_members hgm ON hgm.host_object_id = ohid}
+			${if_table:hg,hgm:left join ${TABLE_PREFIX}hostgroups hg ON hg.hostgroup_id = hgm.hostgroup_id}
+			${if_table:ohg,hg,hgm:left join ${TABLE_PREFIX}objects ohg ON ohgid = hg.hostgroup_object_id}
+			
+			${if_table:sgm:left join ${TABLE_PREFIX}servicegroup_members sgm on sgm.service_object_id = osid}
+			${if_table:sg,sgm:left join ${TABLE_PREFIX}servicegroups sg on sg.servicegroup_id = sgm.servicegroup_id}
+			${if_table:osg,sg,sgm:left join ${TABLE_PREFIX}objects osg on osgid = sg.servicegroup_object_id}
+			
+			${if_table:i,dth:left join ${TABLE_PREFIX}instances i on i.id = dth.instance_id}
+			
+			${FILTER}
+			${GROUPBY}
+			${ORDERBY}
+			${LIMIT}',
+		
+		self::TARGET_DOWNTIME => 
+			'select
+				distinct ${FIELDS}
+			from
+			${TABLE_PREFIX}scheduleddowntime dt
+			
+			left join ${TABLE_PREFIX}objects os ON os.id = dt.object_id
+			left join ${TABLE_PREFIX}services s ON s.service_object_id = os.id
+			${if_table:ss:left join ${TABLE_PREFIX}servicestatus ss on ss.service_object_id = os.id}
+			
+			left join ${TABLE_PREFIX}objects oh ON oh.id = s.host_object_id OR oh.id = dt.object_id
+			${if_table:h:inner join ${TABLE_PREFIX}hosts h on h.host_object_id = oh.id}
+			${if_table:hs:inner join ${TABLE_PREFIX}hoststatus hs on hs.host_object_id = oh.id}
+			
+			${if_table:hgm:left join ${TABLE_PREFIX}hostgroup_members hgm ON hgm.host_object_id = oh.id}
+			${if_table:hg,hgm:left join ${TABLE_PREFIX}hostgroups hg ON hg.hostgroup_id = hgm.hostgroup_id}
+			${if_table:ohg,hg,hgm:left join ${TABLE_PREFIX}objects ohg ON ohg.id = hg.hostgroup_object_id}
+			
+			${if_table:sgm:left join ${TABLE_PREFIX}servicegroup_members sgm on sgm.service_object_id = os.id}
+			${if_table:sg,sgm:left join ${TABLE_PREFIX}servicegroups sg on sg.servicegroup_id = sgm.servicegroup_id}
+			${if_table:osg,sg,sgm:left join ${TABLE_PREFIX}objects osg on osg.id = sg.servicegroup_object_id}
+			
+			${if_table:i,dt:left join ${TABLE_PREFIX}instances i on i.id = dt.instance_id}
+			
+			${FILTER}
+			${GROUPBY}
+			${ORDERBY}
 			${LIMIT}'
 		);
 			// COLUMNS
@@ -472,6 +533,7 @@ class IcingaApiSearchIdoOci8
 		'HOST_ALIAS' => array('h', 'alias'),
 		'HOST_DISPLAY_NAME' => array('h', 'display_name'),
 		'HOST_ADDRESS' => array('h', 'address'),
+		'HOST_ADDRESS6' => array('h', 'address6'),
 		'HOST_ACTIVE_CHECKS_ENABLED' => array('h', 'active_checks_enabled'),
 		'HOST_CONFIG_TYPE' => array('h', 'config_type'),
 		'HOST_FLAP_DETECTION_ENABLED' => array('hs', 'flap_detection_enabled'),
@@ -529,7 +591,7 @@ class IcingaApiSearchIdoOci8
 		'HOST_CHILD_NAME' => array('oh', 'name1'),
 		'HOST_CUSTOMVARIABLE_NAME' => array('cvsh', 'varname'),
 		'HOST_CUSTOMVARIABLE_VALUE' => array('cvsh', 'varvalue'),
-		'HOST_IS_PENDING' => array('hs','has_been_checked','(%s-hs.should_be_scheduled)*-1'),
+		'HOST_IS_PENDING' => array('hs','has_been_checked','(%s-hs.should_be_scheduled)*-1>0'),
 
 		// Service data
 		'SERVICE_ID' => array('s', 'id'),
@@ -585,7 +647,7 @@ class IcingaApiSearchIdoOci8
 		'SERVICE_CUSTOMVARIABLE_NAME' => array('cvss', 'varname'),
 		'SERVICE_CUSTOMVARIABLE_VALUE' => array('cvss', 'varvalue'),
 		'SERVICE_STATE_COUNT' => array('count(ss', 'current_state)'),
-		'SERVICE_IS_PENDING' => array('ss','has_been_checked','(%s-ss.should_be_scheduled)*-1'),
+		'SERVICE_IS_PENDING' => array('ss','has_been_checked','(%s-ss.should_be_scheduled)*-1>0'),
 
 		// Config vars
 		'CONFIG_VAR_ID' => array('cfv', 'id'),
@@ -656,7 +718,46 @@ class IcingaApiSearchIdoOci8
 		'COMMENT_IS_PERSISTENT' => array('co', 'is_persistent'),
 		'COMMENT_SOURCE' => array('co', 'comment_source'),
 		'COMMENT_EXPIRES' => array('co', 'expires'),
-		'COMMENT_EXPIRATION_TIME' => array('co', 'expiration_time')
+		'COMMENT_EXPIRATION_TIME' => array('co', 'expiration_time'),
+		
+			// Downtimehistory
+		'DOWNTIMEHISTORY_ID' => array ('dthh', 'id'),
+		'DOWNTIMEHISTORY_INSTANCE_ID' => array ('dth', 'instance_id'),
+		'DOWNTIMEHISTORY_DOWNTIME_TYPE' => array ('dth', 'downtime_type'),
+		'DOWNTIMEHISTORY_OBJECT_ID' => array ('dth', 'object_id'),
+		'DOWNTIMEHISTORY_ENTRY_TIME' => array ('dth', 'entry_time'),
+		'DOWNTIMEHISTORY_AUTHOR_NAME' => array ('dth', 'author_name'),
+		'DOWNTIMEHISTORY_COMMENT_DATA' => array ('dth', 'comment_data'),
+		'DOWNTIMEHISTORY_INTERNAL_DOWNTIME_ID' => array ('dth', 'internal_downtime_id'),
+		'DOWNTIMEHISTORY_TRIGGERED_BY_ID' => array ('dth', 'triggered_by_id'),
+		'DOWNTIMEHISTORY_IS_FIXED' => array ('dth', 'is_fixed'),
+		'DOWNTIMEHISTORY_DURATION' => array ('dth', 'duration'),
+		'DOWNTIMEHISTORY_SCHEDULED_START_TIME' => array ('dth', 'scheduled_start_time'),
+		'DOWNTIMEHISTORY_SCHEDULED_END_TIME' => array ('dth', 'scheduled_end_time'),
+		'DOWNTIMEHISTORY_WAS_STARTED' => array ('dth', 'was_started'),
+		'DOWNTIMEHISTORY_ACTUAL_START_TIME' => array ('dth', 'actual_start_time'),
+		'DOWNTIMEHISTORY_ACTUAL_START_TIME_USEC' => array ('dth', 'actual_start_time_usec'),
+		'DOWNTIMEHISTORY_ACTUAL_END_TIME' => array ('dth', 'actual_end_time'),
+		'DOWNTIMEHISTORY_ACTUAL_END_TIME_USEC' => array ('dth', 'actual_end_time_usec'),
+		'DOWNTIMEHISTORY_WAS_CANCELLED' => array ('dth', 'was_cancelled'),
+		
+			// Downtime
+		'DOWNTIME_ID' => array ('dt', 'id'),
+		'DOWNTIME_INSTANCE_ID' => array ('dt', 'instance_id'),
+		'DOWNTIME_DOWNTIME_TYPE' => array ('dt', 'downtime_type'),
+		'DOWNTIME_OBJECT_ID' => array ('dt', 'object_id'),
+		'DOWNTIME_ENTRY_TIME' => array ('dt', 'entry_time'),
+		'DOWNTIME_AUTHOR_NAME' => array ('dt', 'author_name'),
+		'DOWNTIME_COMMENT_DATA' => array ('dt', 'comment_data'),
+		'DOWNTIME_INTERNAL_DOWNTIME_ID' => array ('dt', 'internal_downtime_id'),
+		'DOWNTIME_TRIGGERED_BY_ID' => array ('dt', 'triggered_by_id'),
+		'DOWNTIME_IS_FIXED' => array ('dt', 'is_fixed'),
+		'DOWNTIME_DURATION' => array ('dt', 'duration'),
+		'DOWNTIME_SCHEDULED_START_TIME' => array ('dt', 'scheduled_start_time'),
+		'DOWNTIME_SCHEDULED_END_TIME' => array ('dt', 'scheduled_end_time'),
+		'DOWNTIME_WAS_STARTED' => array ('dt', 'was_started'),
+		'DOWNTIME_ACTUAL_START_TIME' => array ('dt', 'actual_start_time'),
+		'DOWNTIME_ACTUAL_START_TIME_USEC' => array ('dt', 'actual_start_time_usec')
 		);
 
 	/*
@@ -757,4 +858,4 @@ class IcingaApiSearchIdoOci8
 
 }
 
-?>
\ No newline at end of file
+?>
