@@ -2,7 +2,7 @@
  * IDO2DB.C - IDO To Database Daemon
  *
  * Copyright (c) 2005-2008 Ethan Galstad
- * Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org)
+ * Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org)
  *
  **************************************************************/
 
@@ -27,10 +27,6 @@
 #include "../../../include/dh.h"
 #endif
 
-#define IDO2DB_NAME "IDO2DB"
-#define IDO2DB_DATE "10-25-2010"
-#define IDO2DB_VERSION "1.2.1"
-
 extern int use_ssl;
 
 extern int errno;
@@ -41,6 +37,18 @@ extern char *ido2db_db_tablenames[IDO2DB_MAX_DBTABLES];
 extern int ido2db_check_dbd_driver(void);
 #endif
 
+/* use global dynamic buffer for mutex locks */
+ido_dbuf dbuf;
+static pthread_mutex_t ido2db_dbuf_lock;
+
+static void *ido2db_thread_cleanup_exit_handler(void *);
+static void *ido2db_thread_worker_exit_handler(void *);
+
+/*
+pthread_mutex_lock(&ido2db_dbuf_lock);
+pthread_mutex_unlock(&ido2db_dbuf_lock);
+*/
+
 #ifdef HAVE_SSL
 SSL_METHOD *meth;
 SSL_CTX *ctx;
@@ -67,7 +75,7 @@ int ido2db_run_foreground=IDO_FALSE;
 
 ido2db_dbconfig ido2db_db_settings;
 ido2db_idi thread_idi;
-pthread_t thread_pool[1];
+pthread_t thread_pool[IDO2DB_NR_OF_THREADS];
 
 time_t ido2db_db_last_checkin_time=0L;
 
@@ -85,8 +93,8 @@ char *sigs[35]={"EXIT","HUP","INT","QUIT","ILL","TRAP","ABRT","BUS","FPE","KILL"
 int ido2db_open_debug_log(void);
 int ido2db_close_debug_log(void);
 
-static void *ido2db_thread_cleanup_exit_handler(void *);
 
+int dummy;	/* reduce compiler warnings */
 
 
 int main(int argc, char **argv){
@@ -116,7 +124,7 @@ int main(int argc, char **argv){
 		printf("\n");
 		printf("%s %s\n",IDO2DB_NAME,IDO2DB_VERSION);
 		printf("Copyright(c) 2005-2008 Ethan Galstad (nagios@nagios.org)\n");
-		printf("Copyright(c) 2009-2010 Icinga Development Team (http://www.icinga.org)\n");
+		printf("Copyright(c) 2009-2011 Icinga Development Team (http://www.icinga.org)\n");
 		printf("Last Modified: %s\n",IDO2DB_DATE);
 		printf("License: GPL v2\n");
 #ifdef HAVE_SSL
@@ -145,7 +153,7 @@ int main(int argc, char **argv){
         }
 
 	/* print starting info to syslog */
-	syslog(LOG_USER | LOG_INFO, "%s %s (%s) Copyright (c) 2005-2008 Ethan Galstad (nagios@nagios.org), Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org))", IDO2DB_NAME, IDO2DB_VERSION, IDO2DB_DATE);
+	syslog(LOG_USER | LOG_INFO, "%s %s (%s) Copyright (c) 2005-2008 Ethan Galstad (nagios@nagios.org), Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org))", IDO2DB_NAME, IDO2DB_VERSION, IDO2DB_DATE);
 	syslog(LOG_USER | LOG_INFO, "%s %s starting... (PID=%d)\n", IDO2DB_NAME, IDO2DB_VERSION, (int)getpid() );
 
 	if (ido2db_socket_type==IDO_SINK_UNIXSOCKET && use_ssl == IDO_TRUE){
@@ -396,8 +404,10 @@ int ido2db_process_config_file(char *filename){
 	int result=IDO_OK;
 
 	/* open the file */
-	if((thefile=ido_mmap_fopen(filename))==NULL)
+	if((thefile=ido_mmap_fopen(filename))==NULL){
+		syslog(LOG_ERR, "Error: Unable to open configuration file %s: %s\n", filename, strerror(errno));
 		return IDO_ERROR;
+	}
 
 	/* process each line of the file */
 	while((buf=ido_mmap_fgets(thefile))){
@@ -522,6 +532,10 @@ int ido2db_process_config_var(char *arg){
 		if((ido2db_db_settings.dbprefix=strdup(val))==NULL)
 			return IDO_ERROR;
 	        }
+	else if(!strcmp(var,"db_socket")){
+		if((ido2db_db_settings.dbsocket=strdup(val))==NULL)
+			return IDO_ERROR;
+	        }
 	else if(!strcmp(var,"max_timedevents_age"))
 		ido2db_db_settings.max_timedevents_age=strtoul(val,NULL,0)*60;
 	else if(!strcmp(var,"max_systemcommands_age"))
@@ -581,6 +595,10 @@ int ido2db_process_config_var(char *arg){
 		ido2db_db_settings.clean_config_tables_on_core_startup=(atoi(val)>0)?IDO_TRUE:IDO_FALSE;
 	}
 
+        else if(!strcmp(var,"oci_errors_to_syslog")){
+                ido2db_db_settings.oci_errors_to_syslog=(atoi(val)>0)?IDO_TRUE:IDO_FALSE;
+	}
+
 	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_process_config_var() end\n");
 
 	return IDO_OK;
@@ -599,6 +617,7 @@ int ido2db_initialize_variables(void){
 	ido2db_db_settings.password=NULL;
 	ido2db_db_settings.dbname=NULL;
 	ido2db_db_settings.dbprefix=NULL;
+	ido2db_db_settings.dbsocket=NULL;
 	ido2db_db_settings.max_timedevents_age=0L;
 	ido2db_db_settings.max_systemcommands_age=0L;
 	ido2db_db_settings.max_servicechecks_age=0L;
@@ -611,6 +630,7 @@ int ido2db_initialize_variables(void){
 	ido2db_db_settings.housekeeping_thread_startup_delay=(unsigned long)DEFAULT_HOUSEKEEPING_THREAD_STARTUP_DELAY; /* set the default if missing in ido2db.cfg */
 	ido2db_db_settings.clean_realtime_tables_on_core_startup=IDO_TRUE; /* default is cleaning on startup */
 	ido2db_db_settings.clean_config_tables_on_core_startup=IDO_TRUE;
+	ido2db_db_settings.oci_errors_to_syslog=DEFAULT_OCI_ERRORS_TO_SYSLOG;
 
 	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_initialize_variables() end\n");
 	return IDO_OK;
@@ -663,6 +683,10 @@ int ido2db_free_program_memory(void){
 		free(ido2db_db_settings.dbprefix);
 		ido2db_db_settings.dbprefix=NULL;
 		}
+	if(ido2db_db_settings.dbsocket){
+		free(ido2db_db_settings.dbsocket);
+		ido2db_db_settings.dbsocket=NULL;
+		}
 	if(ido2db_debug_file){
 		free(ido2db_debug_file);
 		ido2db_debug_file=NULL;
@@ -866,9 +890,9 @@ int ido2db_daemonize(void){
 	if(lock_file){
 		/* write PID to lockfile... */
 		lseek(lockfile,0,SEEK_SET);
-		ftruncate(lockfile,0);
+		dummy=ftruncate(lockfile,0);
 		sprintf(buf,"%d\n",(int)getpid());
-		write(lockfile,buf,strlen(buf));
+		dummy=write(lockfile,buf,strlen(buf));
 
 		/* make sure lock file stays open while program is executing... */
 		val=fcntl(lockfile,F_GETFD,0);
@@ -974,7 +998,8 @@ void ido2db_child_sighandler(int sig){
 		ido2db_free_program_memory();
 	}
 
-	ido2db_kill_threads();
+	/* terminate threads */
+	ido2db_terminate_threads();
 
 	_exit(0);
 
@@ -1149,7 +1174,6 @@ int ido2db_wait_for_connections(void){
 
 
 int ido2db_handle_client_connection(int sd){
-	ido_dbuf dbuf;
 	int dbuf_chunk=2048;
 	ido2db_idi idi;
 	char buf[512];
@@ -1157,6 +1181,9 @@ int ido2db_handle_client_connection(int sd){
 	int error=IDO_FALSE;
 
 	int pthread_ret=0;
+	sigset_t newmask;
+	pthread_attr_t attr;
+
 #ifdef HAVE_SSL
 	SSL *ssl=NULL;
 #endif
@@ -1178,16 +1205,36 @@ int ido2db_handle_client_connection(int sd){
 	signal(SIGSEGV,ido2db_child_sighandler);
 	signal(SIGFPE,ido2db_child_sighandler);
 
+	/* new thread should block all signals */
+	sigfillset(&newmask);
+	pthread_sigmask(SIG_BLOCK,&newmask,NULL);
+
+	/* set stack size */
+	pthread_attr_init(&attr);
+	if(pthread_attr_setstacksize(&attr, IDO2DB_DEFAULT_THREAD_STACK_SIZE)!=0)
+		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_connection() pthread_attr_setstacksize with %lu error\n", IDO2DB_DEFAULT_THREAD_STACK_SIZE);
+
 	/* create cleanup thread */
-	if ((pthread_ret = pthread_create(&thread_pool[0], NULL, ido2db_thread_cleanup, &idi)) != 0) {
+	if ((pthread_ret = pthread_create(&thread_pool[IDO2DB_THREAD_POOL_CLEANER], &attr, ido2db_thread_cleanup, &idi)) != 0) {
 		syslog(LOG_ERR,"Could not create thread... exiting with error '%s'\n", strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
+        /* create worker thread */
+	/*
+        if ((pthread_ret = pthread_create(&thread_pool[IDO2DB_THREAD_POOL_WORKER], &attr, ido2db_thread_worker, &idi)) != 0) {
+                syslog(LOG_ERR,"Could not create thread... exiting with error '%s'\n", strerror(errno));
+                exit(EXIT_FAILURE);
+        }
+	*/
+
+	/* main thread should unblock all signals */
+	pthread_sigmask(SIG_UNBLOCK,&newmask,NULL);
+	pthread_attr_destroy(&attr);
+
 	/* initialize input data information */
 	ido2db_idi_init(&idi);
 
-
 	/* initialize dynamic buffer (2KB chunk size) */
 	ido_dbuf_init(&dbuf,dbuf_chunk);
 
@@ -1202,9 +1249,9 @@ int ido2db_handle_client_connection(int sd){
 			syslog(LOG_USER | LOG_INFO,"Error: database connection failed, forced client disconnect...\n");
 			ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_connection() idi.dbinfo.connected is '%d'\n", idi.dbinfo.connected);
 
-			/* kill sub threads */
-			pthread_kill(thread_pool[0], SIGINT);
-			pthread_cancel(thread_pool[0]);
+			/* terminate threads */
+			terminate_worker_thread();
+			terminate_cleanup_thread();
 
 		        /* free memory allocated to dynamic buffer */
 			ido_dbuf_free(&dbuf);
@@ -1294,10 +1341,17 @@ int ido2db_handle_client_connection(int sd){
 
 		/* append data we just read to dynamic buffer */
 		buf[result]='\x0';
+		/* 2011-02-23 MF: lock dynamic buffer with a mutex when writing */
+		pthread_mutex_lock(&ido2db_dbuf_lock);
 		ido_dbuf_strcat(&dbuf,buf);
+		pthread_mutex_unlock(&ido2db_dbuf_lock);
 
 		/* check for completed lines of input */
-		ido2db_check_for_client_input(&idi,&dbuf, thread_pool);
+		/* 2011-02-23 MF: only do that in a worker thread */
+		/* 2011-05-02 MF: redo it the old way */
+
+		ido2db_check_for_client_input(&idi);
+
 
 		/* should we disconnect the client? */
 		if(idi.disconnect_client==IDO_TRUE){
@@ -1313,14 +1367,9 @@ int ido2db_handle_client_connection(int sd){
 	printf("BYTES: %lu, LINES: %lu\n",idi.bytes_processed,idi.lines_processed);
 #endif
 
-	/* Kill all sub threads */
-	/* Experimental and may be dangerous... */
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_connection() idi.disconnect_client is '%d'\n", idi.disconnect_client);
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_connection() Cancel cleanup thread to terminate\n");
-	// pthread_join(thread_pool[0], NULL);
-	pthread_kill(thread_pool[0], SIGINT);
-	pthread_cancel(thread_pool[0]);
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_connection() cleanup thread terminated\n");
+	/* terminate threads */
+	terminate_worker_thread();
+	terminate_cleanup_thread();
 
 	/* free memory allocated to dynamic buffer */
 	ido_dbuf_free(&dbuf);
@@ -1387,67 +1436,83 @@ int ido2db_idi_init(ido2db_idi *idi){
 
 
 /* checks for single lines of input from a client connection */
-int ido2db_check_for_client_input(ido2db_idi *idi,ido_dbuf *dbuf, pthread_t *thread_pool){
+/* 2011-02-23 MF: called in worker thread */
+/* 2011-05-02 MF: restructured sequential */
+int ido2db_check_for_client_input(ido2db_idi *idi){
 	char *buf=NULL;
 	register int x;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() start\n");
+	//ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() start\n");
 
-	if(dbuf==NULL)
-		return IDO_OK;
-	if(dbuf->buf==NULL)
+/*	if(&dbuf==NULL)
+		return IDO_OK;*/
+	if(dbuf.buf==NULL)
 		return IDO_OK;
+	/* check if buffer full? bail out and tell main to disconnect the client! FIXME */
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() dbuf.size=%lu\n", dbuf.used_size);
+
+	//pthread_mutex_lock(&ido2db_dbuf_lock);
+
+	//ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() ido2db_dbuf_lock start\n");
 
 #ifdef DEBUG_IDO2DB2
-	printf("RAWBUF: %s\n",dbuf->buf);
+	printf("RAWBUF: %s\n",dbuf.buf);
 	printf("  USED1: %lu, BYTES: %lu, LINES: %lu\n",dbuf->used_size,idi->bytes_processed,idi->lines_processed);
 #endif
 
 	/* search for complete lines of input */
-	for(x=0;dbuf->buf[x]!='\x0';x++){
+	for(x=0;dbuf.buf[x]!='\x0';x++){
 
 		/* we found the end of a line */
-		if(dbuf->buf[x]=='\n'){
+		if(dbuf.buf[x]=='\n'){
 
 #ifdef DEBUG_IDO2DB2
 			printf("BUF[%d]='\\n'\n",x);
 #endif
 
 			/* handle this line of input */
-			dbuf->buf[x]='\x0';
-			if((buf=strdup(dbuf->buf))){
-				ido2db_handle_client_input(idi,buf, thread_pool);
+			dbuf.buf[x]='\x0';
+
+			if((buf=strdup(dbuf.buf))){
+
+				ido2db_handle_client_input(idi,buf);
+
 				free(buf);
 				buf=NULL;
 				idi->lines_processed++;
 				idi->bytes_processed+=(x+1);
-			        }
+		        }
 
 			/* shift data back to front of buffer and adjust counters */
-			memmove((void *)&dbuf->buf[0],(void *)&dbuf->buf[x+1],(size_t)((int)dbuf->used_size-x-1));
-			dbuf->used_size-=(x+1);
-			dbuf->buf[dbuf->used_size]='\x0';
+			memmove((void *)&dbuf.buf[0],(void *)&dbuf.buf[x+1],(size_t)((int)dbuf.used_size-x-1));
+			dbuf.used_size-=(x+1);
+			dbuf.buf[dbuf.used_size]='\x0';
 			x=-1;
 #ifdef DEBUG_IDO2DB2
-			printf("  USED2: %lu, BYTES: %lu, LINES: %lu\n",dbuf->used_size,idi->bytes_processed,idi->lines_processed);
+			printf("  USED2: %lu, BYTES: %lu, LINES: %lu\n",dbuf.used_size,idi->bytes_processed,idi->lines_processed);
 #endif
-		        }
-	        }
+		}
+	}
+
+	//pthread_mutex_unlock(&ido2db_dbuf_lock);
+
+	//ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() ido2db_dbuf_lock end\n");
+
+	//ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() end\n");
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_check_for_client_input() end\n");
 	return IDO_OK;
-        }
+}
 
 
 /* handles a single line of input from a client connection */
-int ido2db_handle_client_input(ido2db_idi *idi, char *buf, pthread_t *thread_pool){
+int ido2db_handle_client_input(ido2db_idi *idi, char *buf){
 	char *var=NULL;
 	char *val=NULL;
 	unsigned long data_type_long=0L;
 	int data_type=IDO_DATA_NONE;
 	int input_type=IDO2DB_INPUT_DATA_NONE;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_input() start\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_input(instance_name=%s) start\n", idi->instance_name);
 
 #ifdef DEBUG_IDO2DB2
 	printf("HANDLING: '%s'\n",buf);
@@ -1464,6 +1529,8 @@ int ido2db_handle_client_input(ido2db_idi *idi, char *buf, pthread_t *thread_poo
 	if(buf[0]=='\x0')
 		return IDO_OK;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_input() input_section\n");
+
 	switch(idi->current_input_section){
 
 	case IDO2DB_INPUT_SECTION_NONE:
@@ -1746,12 +1813,14 @@ int ido2db_handle_client_input(ido2db_idi *idi, char *buf, pthread_t *thread_poo
 
 				/* the data type is out of range - throw it out */
 				if(data_type>IDO_MAX_DATA_TYPES){
+					ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_input() line: %lu, type: %d, VAL: %s\n",idi->lines_processed,data_type,(val==NULL)?"":val);
 #ifdef DEBUG_IDO2DB2
 					printf("## DISCARD! LINE: %lu, TYPE: %d, VAL: %s\n",idi->lines_processed,data_type,val);
 #endif
 					break;
 			                }
 
+				ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_handle_client_input() line: %lu, type: %d, VAL: %s\n",idi->lines_processed,data_type,(val==NULL)?"":val);
 #ifdef DEBUG_IDO2DB2
 				printf("LINE: %lu, TYPE: %d, VAL:%s\n",idi->lines_processed,data_type,val);
 #endif
@@ -1838,8 +1907,10 @@ int ido2db_add_input_data_item(ido2db_idi *idi, int type, char *buf){
 	case IDO_DATA_CONTACTNAME:
 	case IDO_DATA_DEPENDENTHOSTNAME:
 	case IDO_DATA_DEPENDENTSERVICEDESCRIPTION:
+	case IDO_DATA_DISPLAYNAME:
 	case IDO_DATA_EMAILADDRESS:
 	case IDO_DATA_HOSTADDRESS:
+	case IDO_DATA_HOSTADDRESS6:
 	case IDO_DATA_HOSTALIAS:
 	case IDO_DATA_HOSTCHECKCOMMAND:
 	case IDO_DATA_HOSTCHECKPERIOD:
@@ -1893,8 +1964,11 @@ int ido2db_add_input_data_item(ido2db_idi *idi, int type, char *buf){
 		break;
 	        }
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_add_input_data_item(%s)\n", (newbuf==NULL)?"":newbuf);
+
 	/* check for errors */
 	if(newbuf==NULL){
+		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_add_input_data_item() allocation error\n");
 #ifdef DEBUG_IDO2DB
 		printf("ALLOCATION ERROR\n");
 #endif
@@ -2314,14 +2388,14 @@ int ido2db_convert_standard_data_elements(ido2db_idi *idi, int *type, int *flags
 
 int ido2db_convert_string_to_int(char *buf, int *i){
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_int() start\n");
-
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_int(%s) start\n", buf);
+
 	*i=atoi(buf);
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_int() end\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_int(%d) end\n", *i);
 	return IDO_OK;
         }
 
@@ -2329,11 +2403,11 @@ int ido2db_convert_string_to_int(char *buf, int *i){
 int ido2db_convert_string_to_float(char *buf, float *f){
 	char *endptr=NULL;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_float() start\n");
-
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_float(%s) start\n", buf);
+
 #ifdef HAVE_STRTOF
 	*f=strtof(buf,&endptr);
 #else
@@ -2346,7 +2420,7 @@ int ido2db_convert_string_to_float(char *buf, float *f){
 	if(errno==ERANGE)
 		return IDO_ERROR;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_float() end\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_float(%f) end\n", *f);
 	return IDO_OK;
         }
 
@@ -2354,10 +2428,11 @@ int ido2db_convert_string_to_float(char *buf, float *f){
 int ido2db_convert_string_to_double(char *buf, double *d){
 	char *endptr=NULL;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_double() start\n");
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_double(%s) start\n", buf);
+
 	*d=strtod(buf,&endptr);
 
 	if(*d==0 && (endptr==buf || errno==ERANGE))
@@ -2365,7 +2440,7 @@ int ido2db_convert_string_to_double(char *buf, double *d){
 	if(errno==ERANGE)
 		return IDO_ERROR;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_double() end\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_double(%lf) end\n", *d);
 	return IDO_OK;
         }
 
@@ -2373,11 +2448,11 @@ int ido2db_convert_string_to_double(char *buf, double *d){
 int ido2db_convert_string_to_long(char *buf, long *l){
 	char *endptr=NULL;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_long() start\n");
-
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_long(%s) start\n", buf);
+
 	*l=strtol(buf,&endptr,0);
 
 	if(*l==LONG_MAX && errno==ERANGE)
@@ -2385,7 +2460,7 @@ int ido2db_convert_string_to_long(char *buf, long *l){
 	if(*l==0L && endptr==buf)
 		return IDO_ERROR;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_long() end\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_long(%l) end\n", *l);
 	return IDO_OK;
         }
 
@@ -2393,18 +2468,18 @@ int ido2db_convert_string_to_long(char *buf, long *l){
 int ido2db_convert_string_to_unsignedlong(char *buf, unsigned long *ul){
 	char *endptr=NULL;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_unsignedlong() start\n");
-
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_unsignedlong(%s) start\n", buf);
+
 	*ul=strtoul(buf,&endptr,0);
 	if(*ul==ULONG_MAX && errno==ERANGE)
 		return IDO_ERROR;
 	if(*ul==0L && endptr==buf)
 		return IDO_ERROR;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_unsignedlong() end\n");
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_unsignedlong(%lu) end\n", *ul);
 	return IDO_OK;
 }
 
@@ -2414,11 +2489,11 @@ int ido2db_convert_string_to_timeval(char *buf, struct timeval *tv){
 	char *ptr=NULL;
 	int result=IDO_OK;
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_timeval() start\n");
-
 	if(buf==NULL)
 		return IDO_ERROR;
 
+	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_convert_string_to_timeval(%s) start\n", buf);
+
 	tv->tv_sec=(time_t)0L;
 	tv->tv_usec=(suseconds_t)0L;
 
@@ -2534,6 +2609,75 @@ int ido2db_log_debug_info(int level, int verbosity, const char *fmt, ...){
 	return IDO_OK;
 	}
 
+
+/********************************************************************
+ *
+ * working on dbuf - this is the function for the buffer reading thread
+ *
+ ********************************************************************/
+
+void * ido2db_thread_worker(void *data) {
+
+        ido2db_idi *idi = (ido2db_idi*) data;
+
+        struct timespec delay;
+        delay.tv_sec = 5;
+        delay.tv_nsec = 500000;
+	nanosleep(&delay, NULL);
+        delay.tv_sec = 0;
+
+        ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_worker() start\n");
+
+	/* specify cleanup routine */
+        pthread_cleanup_push((void *) &ido2db_thread_worker_exit_handler, NULL);
+
+	/* set cancellation info */
+	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
+	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
+
+        while(1){
+
+                /* should we shutdown? */
+                pthread_testcancel();
+
+		/* sleep a bit */
+		nanosleep(&delay, NULL);
+
+                if(idi->disconnect_client==IDO_TRUE){
+                        ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_worker(): origin idi said we should disconnect the client\n");
+                        break;
+                }
+
+		/* check for client input */
+		//ido2db_check_for_client_input(idi);
+
+		/* sleep a bit */
+		nanosleep(&delay, NULL);
+
+		/* should we shutdown? */
+		pthread_testcancel();
+        }
+
+        pthread_cleanup_pop(0);
+
+        ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_worker() end\n");
+
+	pthread_exit((void *) pthread_self());
+}
+
+/* ******************************************************************
+ *
+ * exit_handler_mem is called as thread canceling
+ *
+ * ******************************************************************/
+
+static void *ido2db_thread_worker_exit_handler(void * arg) {
+        ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_worker() cleanup_exit_handler...\n");
+        return 0;
+
+}
+
+
 /********************************************************************
  *
  * housekeeping - this is the function for the db trimming thread
@@ -2544,8 +2688,6 @@ void * ido2db_thread_cleanup(void *data) {
 
 	ido2db_idi *idi = (ido2db_idi*) data;
 
-	int old_thread_state;
-
 	struct timespec delay;
 	delay.tv_sec = 0;
 	delay.tv_nsec = 500;
@@ -2554,23 +2696,27 @@ void * ido2db_thread_cleanup(void *data) {
 	//delay.tv_sec = 60;
 	/* allowed to be set in config */
 	delay.tv_sec = ido2db_db_settings.housekeeping_thread_startup_delay;
+
+	/* the minimum is the default, otherwise overwrite */
+	if(delay.tv_sec<DEFAULT_HOUSEKEEPING_THREAD_STARTUP_DELAY)
+		delay.tv_sec=DEFAULT_HOUSEKEEPING_THREAD_STARTUP_DELAY;
+
 	nanosleep(&delay, NULL);
 
 	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() start\n");
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() initialize thread idi\n");
-
 	/* initialize input data information */
 	ido2db_idi_init(&thread_idi);
 
 	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() initialize thread db connection\n");
+
 	/* initialize database connection */
 	ido2db_db_init(&thread_idi);
 
 	if(ido2db_db_connect(&thread_idi)==IDO_ERROR){
 
 		/* tell main process to disconnect */
-		idi->disconnect_client==IDO_TRUE;
+		idi->disconnect_client=IDO_TRUE;
 
 		/* cleanup the thread */
         	ido2db_db_deinit(&thread_idi);
@@ -2578,21 +2724,23 @@ void * ido2db_thread_cleanup(void *data) {
 	        /* free memory */
 	        ido2db_free_input_memory(&thread_idi);
 	        ido2db_free_connection_memory(&thread_idi);
-		return;
+
+		return (void*)IDO_ERROR;
 	}
 
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() pthread_cleanup push()\n");
+	/* specify cleanup routine */
 	pthread_cleanup_push((void *) &ido2db_thread_cleanup_exit_handler, NULL);
-	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() pthread_cleanup push() end \n");
 
         delay.tv_sec = 0;
         delay.tv_nsec = 500;
 
+	/* keep on looping for an instance name from main thread */
 	while(idi->instance_name==NULL) {
 		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() nanosleeping cause missing instance_name...\n");
 		nanosleep(&delay, NULL);
 	}
 
+
 	/* copy needed idi information */
 	thread_idi.instance_name = idi->instance_name;
 	thread_idi.agent_name = "IDO2DB Trimming Thread";
@@ -2601,8 +2749,8 @@ void * ido2db_thread_cleanup(void *data) {
 	thread_idi.connect_source = idi->connect_source;
 	thread_idi.connect_type = idi->connect_type;
 
-
 	delay.tv_sec = 5;
+
 	/* save connection info to DB */
 	while(ido2db_thread_db_hello(&thread_idi) == IDO_ERROR) {
 		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() no instance found, sleeping...\n");
@@ -2610,28 +2758,22 @@ void * ido2db_thread_cleanup(void *data) {
 	}
 
 	while(1){
+
+		/* should we shutdown? */
+		pthread_testcancel();
+
 		if(idi->disconnect_client==IDO_TRUE){
 			ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup(): origin idi said we should disconnect the client\n");
 			break;
 		}
-		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() working...\n");
-		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() idi is: '%d'\n", idi->disconnect_client);
-		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() has instance name '%s'...\n", thread_idi.instance_name);
-		ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() calling ido2db_db_perform_maintenance...\n");
-
-		/* Set this thread temporary to CANCEL_DISABLE to prevent interuption */
-		if(pthread_setcancelstate( PTHREAD_CANCEL_DISABLE, &old_thread_state) != 0) {
-			ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 1, "ido2db_thread_cleanup() Problem during pthread_setcancelstate() LOCK Cancel\n");
-		}
 
 		/* Perfom DB Maintenance */
 		ido2db_db_perform_maintenance(&thread_idi);
 
-		/* Reset this thread cancelstate */
-		if(pthread_setcancelstate( old_thread_state, NULL) != 0) {
-			ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 1, "ido2db_thread_cleanup() Problem during pthread_setcancelstate() UNLOCK Cancel\n");
-		}
-		sleep(idi->dbinfo.trim_db_interval+1);
+		/* should we shutdown? */
+		pthread_testcancel();
+
+		sleep(thread_idi.dbinfo.trim_db_interval+1);
 	}
 
 	/* gracefully back out of current operation... */
@@ -2643,13 +2785,13 @@ void * ido2db_thread_cleanup(void *data) {
 
         /* free memory */
         ido2db_free_input_memory(&thread_idi);
-        ido2db_free_connection_memory(&thread_idi);
+	ido2db_free_connection_memory(&thread_idi);
 
-	pthread_cleanup_pop(1);
+	pthread_cleanup_pop(0);
 
 	ido2db_log_debug_info(IDO2DB_DEBUGL_PROCESSINFO, 2, "ido2db_thread_cleanup() end\n");
-	pthread_exit((void *) pthread_self());
 
+	pthread_exit((void *) pthread_self());
 }
 
 /* ******************************************************************
@@ -2664,15 +2806,48 @@ static void *ido2db_thread_cleanup_exit_handler(void * arg) {
 
 }
 
-int ido2db_kill_threads(void){
+int ido2db_terminate_threads(void){
 
+	int result;
+
+	/* from cleaner thread */
         ido2db_db_disconnect(&thread_idi);
         ido2db_db_deinit(&thread_idi);
 
-	/* kill sub threads */
-	pthread_kill(thread_pool[0], SIGINT);
-	pthread_cancel(thread_pool[0]);
+	/* terminate each thread on its own */
+	result=terminate_worker_thread();
+	result=terminate_cleanup_thread();
 
 	return IDO_OK;
 }
 
+int terminate_worker_thread(void){
+
+        int result;
+
+        result=pthread_cancel(thread_pool[IDO2DB_THREAD_POOL_WORKER]);
+        /* wait for the worker thread to exit */
+        if(result==0){
+                result=pthread_join(thread_pool[IDO2DB_THREAD_POOL_WORKER],NULL);
+        } /* else only clean memory */
+
+        return IDO_OK;
+
+}
+
+int terminate_cleanup_thread(void){
+
+        int result;
+
+        result=pthread_cancel(thread_pool[IDO2DB_THREAD_POOL_CLEANER]);
+        /* wait for the cleaner thread to exit */
+        if(result==0){
+                result=pthread_join(thread_pool[IDO2DB_THREAD_POOL_CLEANER],NULL);
+        } /* else only clean memory */
+
+        return IDO_OK;
+
+}
+
+
+
