@@ -3,7 +3,8 @@
  * XPDDEFAULT.C - Default performance data routines
  *
  * Copyright (c) 1999-2009 Ethan Galstad (egalstad@nagios.org)
- * Copyright (c) 2009-2010 Icinga Development Team (http://www.icinga.org)
+ * Copyright (c) 2009-2011 Nagios Core Development Team and Community Contributors
+ * Copyright (c) 2009-2011 Icinga Development Team (http://www.icinga.org)
  *
  * License:
  *
@@ -63,15 +64,16 @@ char    *xpddefault_service_perfdata_file_processing_command=NULL;
 command *xpddefault_host_perfdata_file_processing_command_ptr=NULL;
 command *xpddefault_service_perfdata_file_processing_command_ptr=NULL;
 
+int     xpddefault_host_perfdata_process_empty_results=DEFAULT_HOST_PERFDATA_PROCESS_EMPTY_RESULTS;
+int     xpddefault_service_perfdata_process_empty_results=DEFAULT_SERVICE_PERFDATA_PROCESS_EMPTY_RESULTS;
+
 FILE    *xpddefault_host_perfdata_fp=NULL;
 FILE    *xpddefault_service_perfdata_fp=NULL;
 int     xpddefault_host_perfdata_fd=-1;
 int     xpddefault_service_perfdata_fd=-1;
 
-extern char *macro_x[MACRO_X_COUNT];
-
-
-
+static pthread_mutex_t xpddefault_host_perfdata_fp_lock;
+static pthread_mutex_t xpddefault_service_perfdata_fp_lock;
 
 /******************************************************************/
 /***************** COMMON CONFIG INITIALIZATION  ******************/
@@ -194,6 +196,12 @@ int xpddefault_grab_config_directives(char *input){
 	else if(!strcmp(varname,"service_perfdata_file_processing_command"))
 		xpddefault_service_perfdata_file_processing_command=(char *)strdup(varvalue);
 
+	else if(!strcmp(varname,"host_perfdata_process_empty_results"))
+		xpddefault_host_perfdata_process_empty_results=(atoi(varvalue)>0)?TRUE:FALSE;
+
+	else if(!strcmp(varname,"service_perfdata_process_empty_results"))
+		xpddefault_service_perfdata_process_empty_results=(atoi(varvalue)>0)?TRUE:FALSE;
+
 	/* free memory */
 	my_free(varname);
 	my_free(varvalue);
@@ -214,7 +222,9 @@ int xpddefault_initialize_performance_data(char *config_file){
 	char *temp_command_name=NULL;
 	command *temp_command=NULL;
 	time_t current_time;
+	icinga_macros *mac;
 
+	mac = get_global_macros();
 	time(&current_time);
 
 	/* reset vars */
@@ -327,17 +337,17 @@ int xpddefault_initialize_performance_data(char *config_file){
 		schedule_new_event(EVENT_USER_FUNCTION,TRUE,current_time+xpddefault_service_perfdata_file_processing_interval,TRUE,xpddefault_service_perfdata_file_processing_interval,NULL,TRUE,(void *)xpddefault_process_service_perfdata_file,NULL,0);
 
 	/* save the host perf data file macro */
-	my_free(macro_x[MACRO_HOSTPERFDATAFILE]);
+	my_free(mac->x[MACRO_HOSTPERFDATAFILE]);
 	if(xpddefault_host_perfdata_file!=NULL){
-		if((macro_x[MACRO_HOSTPERFDATAFILE]=(char *)strdup(xpddefault_host_perfdata_file)))
-			strip(macro_x[MACRO_HOSTPERFDATAFILE]);
+		if((mac->x[MACRO_HOSTPERFDATAFILE]=(char *)strdup(xpddefault_host_perfdata_file)))
+			strip(mac->x[MACRO_HOSTPERFDATAFILE]);
 	        }
 
 	/* save the service perf data file macro */
-	my_free(macro_x[MACRO_SERVICEPERFDATAFILE]);
+	my_free(mac->x[MACRO_SERVICEPERFDATAFILE]);
 	if(xpddefault_service_perfdata_file!=NULL){
-		if((macro_x[MACRO_SERVICEPERFDATAFILE]=(char *)strdup(xpddefault_service_perfdata_file)))
-			strip(macro_x[MACRO_SERVICEPERFDATAFILE]);
+		if((mac->x[MACRO_SERVICEPERFDATAFILE]=(char *)strdup(xpddefault_service_perfdata_file)))
+			strip(mac->x[MACRO_SERVICEPERFDATAFILE]);
 	        }
 
 	/* free memory */
@@ -345,7 +355,7 @@ int xpddefault_initialize_performance_data(char *config_file){
 	my_free(buffer);
 
 	return OK;
-        }
+}
 
 
 
@@ -379,27 +389,87 @@ int xpddefault_cleanup_performance_data(char *config_file){
 /* updates service performance data */
 int xpddefault_update_service_performance_data(service *svc){
 
+	icinga_macros mac;
+	host *hst;
+
+	/*
+	 * bail early if we've got nothing to do so we don't spend a lot
+	 * of time calculating macros that never get used
+	 * on distributed setups, empty perfdata results are required, so
+	 * only drop out if demanded via configs.
+	 */
+	if(xpddefault_service_perfdata_process_empty_results==FALSE){
+		if (!svc || !svc->perf_data || !*svc->perf_data){
+			return OK;
+		}
+		if ((!xpddefault_service_perfdata_fp || !xpddefault_service_perfdata_file_template) && !xpddefault_service_perfdata_command){
+			return OK;
+		}
+	}
+
+	/*
+	 * we know we've got some work to do, so grab the necessary
+	 * macros and get busy
+	 */
+	memset(&mac, 0, sizeof(mac));
+	hst = find_host(svc->host_name);
+	grab_host_macros_r(&mac, hst);
+	grab_service_macros_r(&mac, svc);
+
 	/* run the performance data command */
-	xpddefault_run_service_performance_data_command(svc);
+	xpddefault_run_service_performance_data_command(&mac, svc);
+
+	/* get rid of used memory we won't need anymore */
+	clear_argv_macros_r(&mac);
 
 	/* update the performance data file */
-	xpddefault_update_service_performance_data_file(svc);
+	xpddefault_update_service_performance_data_file(&mac, svc);
+
+	/* now free() it all */
+	clear_volatile_macros_r(&mac);
 
 	return OK;
-        }
+}
 
 
 /* updates host performance data */
 int xpddefault_update_host_performance_data(host *hst){
 
+	icinga_macros mac;
+
+	/*
+	 * bail early if we've got nothing to do so we don't spend a lot
+	 * of time calculating macros that never get used
+	 * on distributed setups, empty perfdata results are required, so
+	 * only drop out if demanded via configs.
+	 */
+        if(xpddefault_host_perfdata_process_empty_results==FALSE){
+		if (!hst || !hst->perf_data || !*hst->perf_data) {
+			return OK;
+		}
+		if ((!xpddefault_host_perfdata_fp || !xpddefault_host_perfdata_file_template) && !xpddefault_host_perfdata_command) {
+			return OK;
+		}
+	}
+
+	/* set up macros and get to work */
+	memset(&mac, 0, sizeof(mac));
+	grab_host_macros_r(&mac, hst);
+
 	/* run the performance data command */
-	xpddefault_run_host_performance_data_command(hst);
+	xpddefault_run_host_performance_data_command(&mac, hst);
+
+	/* no more commands to run, so we won't need this any more */
+	clear_argv_macros_r(&mac);
 
 	/* update the performance data file */
-	xpddefault_update_host_performance_data_file(hst);
+	xpddefault_update_host_performance_data_file(&mac, hst);
+
+	/* free() all */
+	clear_volatile_macros_r(&mac);
 
 	return OK;
-        }
+}
 
 
 
@@ -410,15 +480,15 @@ int xpddefault_update_host_performance_data(host *hst){
 
 
 /* runs the service performance data command */
-int xpddefault_run_service_performance_data_command(service *svc){
+int xpddefault_run_service_performance_data_command(icinga_macros *mac, service *svc){
 	char *raw_command_line=NULL;
 	char *processed_command_line=NULL;
-	host *temp_host;
 	int early_timeout=FALSE;
 	double exectime;
 	int result=OK;
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
 
+
 	log_debug_info(DEBUGL_FUNCTIONS,0,"run_service_performance_data_command()\n");
 
 	if(svc==NULL)
@@ -428,30 +498,22 @@ int xpddefault_run_service_performance_data_command(service *svc){
 	if(xpddefault_service_perfdata_command==NULL)
 		return OK;
 
-	/* find the associated host */
-	temp_host=find_host(svc->host_name);
-
-	/* update service macros */
-	clear_volatile_macros();
-	grab_host_macros(temp_host);
-	grab_service_macros(svc);
-
 	/* get the raw command line */
-	get_raw_command_line(xpddefault_service_perfdata_command_ptr,xpddefault_service_perfdata_command,&raw_command_line,macro_options);
+	get_raw_command_line_r(mac, xpddefault_service_perfdata_command_ptr,xpddefault_service_perfdata_command,&raw_command_line,macro_options);
 	if(raw_command_line==NULL)
 		return ERROR;
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw service performance data command line: %s\n",raw_command_line);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,&processed_command_line,macro_options);
+	process_macros_r(mac, raw_command_line,&processed_command_line,macro_options);
 	if(processed_command_line==NULL)
 		return ERROR;
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed service performance data command line: %s\n",processed_command_line);
 
 	/* run the command */
-	my_system(processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+	my_system_r(mac, processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
 
 	/* check to see if the command timed out */
 	if(early_timeout==TRUE)
@@ -462,11 +524,11 @@ int xpddefault_run_service_performance_data_command(service *svc){
 	my_free(processed_command_line);
 
 	return result;
-        }
+}
 
 
 /* runs the host performance data command */
-int xpddefault_run_host_performance_data_command(host *hst){
+int xpddefault_run_host_performance_data_command(icinga_macros *mac, host *hst){
 	char *raw_command_line=NULL;
 	char *processed_command_line=NULL;
 	int early_timeout=FALSE;
@@ -483,24 +545,20 @@ int xpddefault_run_host_performance_data_command(host *hst){
 	if(xpddefault_host_perfdata_command==NULL)
 		return OK;
 
-	/* update host macros */
-	clear_volatile_macros();
-	grab_host_macros(hst);
-
 	/* get the raw command line */
-	get_raw_command_line(xpddefault_host_perfdata_command_ptr,xpddefault_host_perfdata_command,&raw_command_line,macro_options);
+	get_raw_command_line_r(mac, xpddefault_host_perfdata_command_ptr,xpddefault_host_perfdata_command,&raw_command_line,macro_options);
 	if(raw_command_line==NULL)
 		return ERROR;
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw host performance data command line: %s\n",raw_command_line);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,&processed_command_line,macro_options);
+	process_macros_r(mac, raw_command_line,&processed_command_line,macro_options);
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed host performance data command line: %s\n",processed_command_line);
 
 	/* run the command */
-	my_system(processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+	my_system_r(mac, processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
 	if(processed_command_line==NULL)
 		return ERROR;
 
@@ -513,7 +571,7 @@ int xpddefault_run_host_performance_data_command(host *hst){
 	my_free(processed_command_line);
 
 	return result;
-        }
+}
 
 
 
@@ -643,10 +701,9 @@ int xpddefault_preprocess_file_templates(char *template){
 
 
 /* updates service performance data file */
-int xpddefault_update_service_performance_data_file(service *svc){
+int xpddefault_update_service_performance_data_file(icinga_macros *mac, service *svc){
 	char *raw_output=NULL;
 	char *processed_output=NULL;
-	host *temp_host=NULL;
 	int result=OK;
 
 	log_debug_info(DEBUGL_FUNCTIONS,0,"update_service_performance_data_file()\n");
@@ -658,41 +715,35 @@ int xpddefault_update_service_performance_data_file(service *svc){
 	if(xpddefault_service_perfdata_fp==NULL || xpddefault_service_perfdata_file_template==NULL)
 		return OK;
 
-	/* find the associated host */
-	temp_host=find_host(svc->host_name);
-
-	/* update service macros */
-	clear_volatile_macros();
-	grab_host_macros(temp_host);
-	grab_service_macros(svc);
-
 	/* get the raw line to write */
 	raw_output=(char *)strdup(xpddefault_service_perfdata_file_template);
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw service performance data file output: %s\n",raw_output);
 
 	/* process any macros in the raw output line */
-	process_macros(raw_output,&processed_output,0);
+	process_macros_r(mac, raw_output, &processed_output, 0);
 	if(processed_output==NULL)
 		return ERROR;
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed service performance data file output: %s\n",processed_output);
 
-	/* write the processed output line containing performance data to the service perfdata file */
-	fputs(processed_output,xpddefault_service_perfdata_fp);
-	fputs("\n",xpddefault_service_perfdata_fp);
-	fflush(xpddefault_service_perfdata_fp);
+	/* lock, write to and unlock host performance data file */
+	pthread_mutex_lock(&xpddefault_service_perfdata_fp_lock);
+ 	fputs(processed_output,xpddefault_service_perfdata_fp);
+	fputc('\n',xpddefault_service_perfdata_fp);
+ 	fflush(xpddefault_service_perfdata_fp);
+	pthread_mutex_unlock(&xpddefault_service_perfdata_fp_lock);
 
 	/* free memory */
 	my_free(raw_output);
 	my_free(processed_output);
 
 	return result;
-        }
+}
 
 
 /* updates host performance data file */
-int xpddefault_update_host_performance_data_file(host *hst){
+int xpddefault_update_host_performance_data_file(icinga_macros *mac, host *hst){
 	char *raw_output=NULL;
 	char *processed_output=NULL;
 	int result=OK;
@@ -706,26 +757,24 @@ int xpddefault_update_host_performance_data_file(host *hst){
 	if(xpddefault_host_perfdata_fp==NULL || xpddefault_host_perfdata_file_template==NULL)
 		return OK;
 
-	/* update host macros */
-	clear_volatile_macros();
-	grab_host_macros(hst);
-
 	/* get the raw output */
 	raw_output=(char *)strdup(xpddefault_host_perfdata_file_template);
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw host performance file output: %s\n",raw_output);
 
 	/* process any macros in the raw output */
-	process_macros(raw_output,&processed_output,0);
+	process_macros_r(mac, raw_output,&processed_output,0);
 	if(processed_output==NULL)
 		return ERROR;
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed host performance data file output: %s\n",processed_output);
 
-	/* write the processed output line containing performance data to the host perfdata file */
-	fputs(processed_output,xpddefault_host_perfdata_fp);
-	fputs("\n",xpddefault_host_perfdata_fp);
-	fflush(xpddefault_host_perfdata_fp);
+	/* lock, write to and unlock host performance data file */
+	pthread_mutex_lock(&xpddefault_host_perfdata_fp_lock);
+	fputs(processed_output, xpddefault_host_perfdata_fp);
+	fputc('\n', xpddefault_host_perfdata_fp);
+ 	fflush(xpddefault_host_perfdata_fp);
+	pthread_mutex_unlock(&xpddefault_host_perfdata_fp_lock);
 
 	/* free memory */
 	my_free(raw_output);
@@ -743,7 +792,7 @@ int xpddefault_process_host_perfdata_file(void){
 	double exectime=0.0;
 	int result=OK;
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
-
+	icinga_macros mac;
 
 	log_debug_info(DEBUGL_FUNCTIONS,0,"process_host_perfdata_file()\n");
 
@@ -751,42 +800,49 @@ int xpddefault_process_host_perfdata_file(void){
 	if(xpddefault_host_perfdata_file_processing_command==NULL)
 		return OK;
 
-	/* close the performance data file */
-	xpddefault_close_host_perfdata_file();
-
-	/* update macros */
-	clear_volatile_macros();
+	/* init macros */
+	memset(&mac, 0, sizeof(mac));
 
 	/* get the raw command line */
-	get_raw_command_line(xpddefault_host_perfdata_file_processing_command_ptr,xpddefault_host_perfdata_file_processing_command,&raw_command_line,macro_options);
-	if(raw_command_line==NULL)
+	get_raw_command_line_r(&mac, xpddefault_host_perfdata_file_processing_command_ptr,xpddefault_host_perfdata_file_processing_command,&raw_command_line,macro_options);
+	if(raw_command_line==NULL) {
+		clear_volatile_macros_r(&mac);
 		return ERROR;
+	}
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw host performance data file processing command line: %s\n",raw_command_line);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,&processed_command_line,macro_options);
-	if(processed_command_line==NULL)
+	process_macros_r(&mac, raw_command_line,&processed_command_line,macro_options);
+	if(processed_command_line==NULL) {
+		clear_volatile_macros_r(&mac);
 		return ERROR;
+	}
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed host performance data file processing command line: %s\n",processed_command_line);
 
+	/* lock and close the performance data file */
+	pthread_mutex_lock(&xpddefault_host_perfdata_fp_lock);
+	xpddefault_close_host_perfdata_file();
+
 	/* run the command */
-	my_system(processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+	my_system_r(&mac, processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+	clear_volatile_macros_r(&mac);
+
+	/* re-open and unlock the performance data file */
+	xpddefault_open_host_perfdata_file();
+	pthread_mutex_unlock(&xpddefault_host_perfdata_fp_lock);
 
 	/* check to see if the command timed out */
 	if(early_timeout==TRUE)
 		logit(NSLOG_RUNTIME_WARNING,TRUE,"Warning: Host performance data file processing command '%s' timed out after %d seconds\n",processed_command_line,xpddefault_perfdata_timeout);
 
-	/* re-open the performance data file */
-	xpddefault_open_host_perfdata_file();
-
 	/* free memory */
 	my_free(raw_command_line);
 	my_free(processed_command_line);
 
 	return result;
-        }
+}
 
 
 /* periodically process the service perf data file */
@@ -797,7 +853,7 @@ int xpddefault_process_service_perfdata_file(void){
 	double exectime=0.0;
 	int result=OK;
 	int macro_options=STRIP_ILLEGAL_MACRO_CHARS|ESCAPE_MACRO_CHARS;
-
+	icinga_macros mac;
 
 	log_debug_info(DEBUGL_FUNCTIONS,0,"process_service_perfdata_file()\n");
 
@@ -805,36 +861,44 @@ int xpddefault_process_service_perfdata_file(void){
 	if(xpddefault_service_perfdata_file_processing_command==NULL)
 		return OK;
 
-	/* close the performance data file */
-	xpddefault_close_service_perfdata_file();
-
-	/* update macros */
-	clear_volatile_macros();
+	/* init macros */
+	memset(&mac, 0, sizeof(mac));
 
 	/* get the raw command line */
-	get_raw_command_line(xpddefault_service_perfdata_file_processing_command_ptr,xpddefault_service_perfdata_file_processing_command,&raw_command_line,macro_options);
-	if(raw_command_line==NULL)
+	get_raw_command_line_r(&mac, xpddefault_service_perfdata_file_processing_command_ptr,xpddefault_service_perfdata_file_processing_command,&raw_command_line,macro_options);
+	if(raw_command_line==NULL) {
+		clear_volatile_macros_r(&mac);
 		return ERROR;
+	}
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Raw service performance data file processing command line: %s\n",raw_command_line);
 
 	/* process any macros in the raw command line */
-	process_macros(raw_command_line,&processed_command_line,macro_options);
-	if(processed_command_line==NULL)
+	process_macros_r(&mac, raw_command_line,&processed_command_line,macro_options);
+	if(processed_command_line==NULL) {
+		clear_volatile_macros_r(&mac);
 		return ERROR;
+	}
 
 	log_debug_info(DEBUGL_PERFDATA,2,"Processed service performance data file processing command line: %s\n",processed_command_line);
 
+	/* lock and close the performance data file */
+	pthread_mutex_lock(&xpddefault_service_perfdata_fp_lock);
+	xpddefault_close_service_perfdata_file();
+
 	/* run the command */
-	my_system(processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+	my_system_r(&mac, processed_command_line,xpddefault_perfdata_timeout,&early_timeout,&exectime,NULL,0);
+
+	/* re-open and unlock the performance data file */
+	xpddefault_open_service_perfdata_file();
+	pthread_mutex_unlock(&xpddefault_service_perfdata_fp_lock);
+
+	clear_volatile_macros_r(&mac);
 
 	/* check to see if the command timed out */
 	if(early_timeout==TRUE)
 		logit(NSLOG_RUNTIME_WARNING,TRUE,"Warning: Service performance data file processing command '%s' timed out after %d seconds\n",processed_command_line,xpddefault_perfdata_timeout);
 
-	/* re-open the performance data file */
-	xpddefault_open_service_perfdata_file();
-
 	/* free memory */
 	my_free(raw_command_line);
 	my_free(processed_command_line);
